c-----------------------------------------------------------------------
      program calno2
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 230120              MAIN
c               C. DesAutels Exponent Inc.
c
c --- PURPOSE:  Read CALPUFF dataset 2.1 or later output files (e.g.,
c               CONC.DAT) and calculate NO2 concentrations based on 
c               ARM or OLM methodology. Resulting NO2 concentrations
c               are written to a new CALPUFF concentration file with 
c               NO2 substituted for NOx.
c Updates
c --- 7.0.0 to 7.0.1
c          220403 - 230120 : Updated calculated variable definitions to 
c                            double precision to avoid overflow
c
c  EXTERNAL ROUTINES: GETCMD, CONTROL, HD_CFG, HD_PROC, HD_QA,
c                     HD_OUT, HD_PER, SETPER, COMP, GETMAX,
c                     PUTMAX, OERMAX, PUTPER,
c                     ALLOC_1, ALLOC_2, ALLOC_3, DEALLOC
c-----------------------------------------------------------------------
c --- Access module data
      use header
      use userinf
      use armdata

      implicit none

      integer :: ictrl,iolst,ioinp,iobin,ioz,i
      parameter(ictrl=10,iolst=11,iobin=14,ioz=15,ioinp=20)

      character(len=12)   :: cversion,clevel
      character(len=256)  :: cfname

      integer :: ione,ierr,ifile,file1

      data ione/1/

c --- Identify version and level of CALNO2:
      cversion='7.0.1'
      clevel='230120'

c --- Control file name (default or specified on command line)
      cfname='CALNO2.INP'
      call GETCMD(cfname)

c --- Open and process control file
      open(ictrl,file=cfname,status='old')
      call CONTROL(ictrl,iolst,cversion,clevel)
      close(ictrl)

c --- CALPUFF binary file
      write(iolst,*)'Creating   CALPUFF File: '//TRIM(binfile)
      write(iolst,*)
      write(*,*)'Creating   CALPUFF File: '//TRIM(binfile)
      write(*,*)
      open(iobin,file=binfile,status='unknown',
     &              form='unformatted')

c --- Open CALPUFF input files (No Limit in LF95)
      do ifile=1,nfiles
         ioinparray(ifile)=ioinp+ifile
         write(*,*)'Processing CALPUFF File: '//TRIM(inpfile(ifile))
         write(iolst,*)'Processing File: '//TRIM(inpfile(ifile))
         open(ioinparray(ifile),file=inpfile(ifile),status='old',
     &              form='unformatted')
      enddo

c --- Allocate arrays that depend only on the number of
c --- concurrent CALPUFF files, plus 1 output file
      call ALLOC_1(nfiles+1)

c --- Process top of file header for model run configuration
      do ifile=1,nfiles
         call HD_CFG(iolst,ioinparray(ifile),iobin,ifile,
     &               cversion,clevel,ierr)
         if(ierr.NE.0) stop 'HALTED at call to HD_CFG -- see list file'
      enddo

c --- Allocate remaining arrays for header
      call ALLOC_2(nfiles+1)

c --- Process rest of file header
      do ifile=1,nfiles
         call HD_PROC(ioinparray(ifile),iobin,iolst,ifile)
      enddo

c --- Do selective QA on headers
      call HD_QA(iolst)

c --- Write header data to binary CALPUFF file
      call HD_OUT(iobin,nfiles+1)

c --- Process data records for nfiles using either OLM or ARM
      if (mode.eq.1) then
       call COMPARM(iolst,iobin,ierr)
       if(ierr.NE.0) stop 'HALTED at call to COMPARM --- see list file'

      elseif (mode.eq.2) then
       call COMPOLM(iolst,iobin,ioz,ierr)
       if(ierr.NE.0) stop 'HALTED at call to COMPOLM --- see list file'

      else
       write(*,*)'Error in CONTROL: MODE must be either 1 or 2'
       write(*,*)'MODE = ',mode
                  stop     
      endif
      stop
      end
c---------------------------
c --- Add utility file
      include 'utils.for'
c---------------------------

c-----------------------------------------------------------------------
      subroutine control(ictrl,iolst,cversion,clevel)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403           CONTROL
c               C. DesAutels, Exponent Inc.
c
c --- PURPOSE:  Obtain program control information
c
c --- UPDATES:
c
c --- INPUT:
c               ICTRL    - integer    - Unit number of control file
c               IOLST    - integer    - Unit number of output list file
c               CVERSION - character  - Unit number of output list file
c               CLEVEL   - character  - Unit number of output list file
c
c --- OUTPUT:
c     Module /USERINF/
c
c --- CONTROL called by: (main)
c --- CONTROL calls:     DELIM, MK_CASE, RM_BLANK
c----------------------------------------------------------------------
c --- Access module data
      use userinf
      use armdata

      implicit none

c --- Declare arguments
      integer :: ictrl,iolst
      character(len=*)  ::  cversion,clevel

c --- Locals
      logical :: lprocess
      integer :: i,ieof,ierr, icase, k
      integer :: id1,id2,ieq,isrc
      integer :: l1,l2,r1,r2, num

      character(len=320)  ::  blank320,aline320
      character(len=5)    ::  ulcase(2)
      character(len=1)    ::  dlim

      data dlim/'!'/
      data ulcase/'UPPER','LOWER'/

c --- Establish initial values (others in module userinf)
      isrc=1
      do i=1,320
         blank320(i:i)=' '
      enddo
      lstfile=blank320(1:256)
      lstfile(1:10)='calmax.lst'

c --- Scan control file for the list-file name, and
c --- number of input files and number of sources (must allocate storage arrays)
c -----------------------------------------------------------
c --- (List file is not opened yet so error-writes will go to
c --- a FORT-file)
      nfiles=0
      ieof=0
      do while (ieof.EQ.0)
         aline320=blank320
         read(ictrl,'(a320)',iostat=ieof) aline320
         call DELIM(dlim,aline320,id1,id2,ieq,lprocess)
         if(lprocess) then
c ---       Set range for variable name on left and value on right
            l1=id1+1
            l2=ieq-1
            r1=ieq+1
            r2=id2-1
            num=r2-r1+1
c ---       Convert to upper case
            call MK_CASE(iolst,'UPPER',aline320,ierr)
            if(ierr.NE.0) stop 'HALTED in CONTROL'
c ---       Interpret subset of inputs
            if(INDEX(aline320(l1:l2),'INPFILE').GT.0) then
               nfiles=nfiles+1
            elseif(INDEX(aline320(l1:l2),'NSOURCE').GT.0) then
               read(aline320(r1:r2),'(i)') nsource
               
            elseif(INDEX(aline320(l1:l2),'LSTFILE').GT.0) then
               if(num.GT.256) then
                  write(*,*)'Error in CONTROL: file pathname '//
     &                      'exceeds 256 characters' 
                  write(*,*) TRIM(aline320)
                  stop
               endif
               lstfile=blank320(1:256)
               lstfile=aline320(r1:r2)
c ---          Left-justify
               lstfile=ADJUSTL(lstfile)
            elseif(INDEX(aline320(l1:l2),'LCFILES').GT.0) then
               if(INDEX(aline320(r1:r2),'T').GT.0) then
                  LCFILES=.TRUE.
               else
                  LCFILES=.FALSE.
               endif
            endif
         endif
      enddo

c --- Set pointer for ULCASE (1=upper, 2=lower)
      icase=1
      if(lcfiles) icase=2

c --- Open list file and write header
      call MK_CASE(iolst,ulcase(icase),lstfile,ierr)
      if(ierr.NE.0) stop 'HALTED in CONTROL'
      open(iolst,file=lstfile,status='unknown')
      write(iolst,*)
      write(iolst,*)
      write(iolst,*)'                    '//
     &              '      CALNO2 MODEL RUN SUMMARY'
      write(iolst,*)'                   VERSION:'//cversion//
     &              '   LEVEL:'//clevel
      write(iolst,*)
      write(iolst,*)

c --- Rewind control file to complete the processing
      REWIND(ictrl)

c --- Allocate arrays for storing/using input file names
      ALLOCATE (inpfile(nfiles),ioinparray(nfiles))
      ALLOCATE (insource(nsource))
      ALLOCATE (no2nox(nsource))

c --- Process control file again for all contents
c ------------------------------------------------
      nfiles=0
      ieof=0
      do while (ieof.EQ.0)
         aline320=blank320
         read(ictrl,'(a320)',iostat=ieof) aline320
         call DELIM(dlim,aline320,id1,id2,ieq,lprocess)
         if(lprocess) then
c ---       Set range for variable name on left and value on right
            l1=id1+1
            l2=ieq-1
            r1=ieq+1
            r2=id2-1
            num=r2-r1+1
c ---       Convert to upper case
            call MK_CASE(iolst,'UPPER',aline320,ierr)
            if(ierr.NE.0) stop 'HALTED in CONTROL -- see list file'
c ---       Check list of expected inputs

C ---   READ LIST OF CALPUFF CONCENTRATION FILES
            if(INDEX(aline320(l1:l2),'INPFILE').GT.0) then
               if(num.GT.256) then
                  write(*,*)'Error in CONTROL: file pathname '//
     &                      'exceeds 256 characters' 
                  write(*,*) TRIM(aline320)
                  stop
               endif
               nfiles=nfiles+1
               inpfile(nfiles)=blank320(1:256)
               inpfile(nfiles)=aline320(r1:r2)
               inpfile(nfiles)=ADJUSTL(inpfile(nfiles))
c ---          Adjust case
               call MK_CASE(iolst,ulcase(icase),inpfile(nfiles),ierr)
               if(ierr.NE.0) stop 'HALTED in CONTROL -- see list file'

C ---      CALPUFF CONCENTRATION OUTPUT FILE
            elseif(INDEX(aline320(l1:l2),'BINFILE').GT.0) then
               if(num.GT.256) then
                  write(*,*)'Error in CONTROL: file pathname '//
     &                      'exceeds 256 characters' 
                  write(*,*) TRIM(aline320)
                  stop
               endif
               binfile=blank320(1:256)
               binfile=aline320(r1:r2)
               binfile=ADJUSTL(binfile)
c ---          Adjust case
               call MK_CASE(iolst,ulcase(icase),binfile,ierr)
               if(ierr.NE.0) stop 'HALTED in CONTROL -- see list file'

C ---     BACKGROUND OZONE FILE
            elseif(INDEX(aline320(l1:l2),'OZFILE').GT.0) then
               if(num.GT.256) then
                  write(*,*)'Error in CONTROL: file pathname '//
     &                      'exceeds 256 characters' 
                  write(*,*) TRIM(aline320)
                  stop
               endif
               ozfile=blank320(1:256)
               ozfile=aline320(r1:r2)
               ozfile=ADJUSTL(ozfile)
c ---          Adjust case
               call MK_CASE(iolst,ulcase(icase),ozfile,ierr)
               if(ierr.NE.0) stop 'HALTED in CONTROL -- see list file'

            elseif(INDEX(aline320(l1:l2),'LSTFILE').GT.0) then
c ---          Nothing:  already processed

            elseif(INDEX(aline320(l1:l2),'LCFILES').GT.0) then
c ---          Nothing:  already processed

            elseif(INDEX(aline320(l1:l2),'NSOURCE').GT.0) then
c ---          Nothing:  already processed

            else
c ---          Remove blanks and reset pointers for rest
               call RM_BLANK(aline320)
               call DELIM(dlim,aline320,id1,id2,ieq,lprocess)
c ---          Set range for variable name on left and value on right
               l1=id1+1
               l2=ieq-1
               r1=ieq+1
               r2=id2-1

               if(INDEX(aline320(l1:l2),'MODE').GT.0) then
                  read(aline320(r1:r2),'(i)') mode

               elseif(INDEX(aline320(l1:l2),'OCOMP').GT.0) then
                  read(aline320(r1:r2),'(i)') ocomp
                  if(ocomp.EQ.1) then
                     locomp=.TRUE.
                  else
                     locomp=.FALSE.
                  endif

               elseif(INDEX(aline320(l1:l2),'OZSRC').GT.0) then
                  read(aline320(r1:r2),'(i)') ozsrc

               elseif(INDEX(aline320(l1:l2),'APROF').GT.0) then
                  read(aline320(r1:r2),'(i)') aprof

C --- READ SOURCE VARIABLES
               elseif(INDEX(aline320(l1:l2),'NO2NOX').GT.0) then
                 if (mode.eq.2) then
                  read(aline320(r1:r2),*)insource(isrc),no2nox(isrc)
                  do k=1,isrc-1
                    if(insource(k).eq.insource(isrc)) then
                       write(*,*)'Stopping Input Error'
                       write(*,*)'See CALNO2 List File'
                       write(iolst,*)'Error: Source Name Repeated:  ',
     &                                   insource(k)
                       stop
                    endif
                  enddo
                  isrc=isrc+1
                 endif
               elseif(INDEX(aline320(l1:l2),'OZJAN').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(1)
               elseif(INDEX(aline320(l1:l2),'OZFEB').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(2)
               elseif(INDEX(aline320(l1:l2),'OZMAR').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(3)
               elseif(INDEX(aline320(l1:l2),'OZAPR').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(4)
               elseif(INDEX(aline320(l1:l2),'OZMAY').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(5)
               elseif(INDEX(aline320(l1:l2),'OZJUN').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(6)
               elseif(INDEX(aline320(l1:l2),'OZJUL').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(7)
               elseif(INDEX(aline320(l1:l2),'OZAUG').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(8)
               elseif(INDEX(aline320(l1:l2),'OZSEP').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(9)
               elseif(INDEX(aline320(l1:l2),'OZOCT').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(10)
               elseif(INDEX(aline320(l1:l2),'OZNOV').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(11)
               elseif(INDEX(aline320(l1:l2),'OZDEC').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') ozmonth(12)             
               elseif(INDEX(aline320(l1:l2),'AFACT').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') afact
               elseif(INDEX(aline320(l1:l2),'BFACT').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') bfact
               elseif(INDEX(aline320(l1:l2),'EQUIL').GT.0) then
                  read(aline320(r1:r2),'(f6.2)') equil
               else
                  write(iolst,*)'Error in CALNO2 control file'
                  write(iolst,*)'Assignment variable unknown:'
                  write(iolst,*)TRIM(aline320)
                  stop 'HALTED in CONTROL -- see list file'
               endif
            endif
         endif
      enddo

      write(iolst,*)
      write(iolst,*)'SUMMARY OF CONTROL FILE OPTIONS SELECTED'
      write(iolst,*)'----------------------------------------'
      write(iolst,*)

      write(iolst,*)'SECTION 1: Processing Options'
      write(iolst,*)
      write(iolst,*)'Number of sources to be processed:'
      write(iolst,*)'NSORUCE = ',nsource
      write(iolst,*)
      write(iolst,*)'Processing Mode:'
      write(iolst,*)'MODE = ',mode
      if(mode.eq.1) then
        write(iolst,*)'Ambient Ratio Method Selected:'
        write(iolst,*)
        write(iolst,*)'Section 2: ARM Options'
        write(iolst,*)
        write(iolst,*)'  Profile Method Selected APROF = ',aprof
        if(aprof.eq.1) then
          write(iolst,*)'  Polynomial Factors: C0 = ',C0_All,
     &   ' C1 = ',C1_All,' C2 = ',C2_All,' C3 = ',C3_All,
     &   ' C4 = ',C4_All,' C5 = ',C5_All,' C6 = ',C6_All
        elseif(aprof.eq.2) then
          write(iolst,*)'  Polynomial Factors: C0 = ',C0_Urb,
     &   ' C1 = ',C1_Urb,' C2 = ',C2_Urb,' C3 = ',C3_Urb,
     &   ' C4 = ',C4_Urb,' C5 = ',C5_Urb,' C6 = ',C6_Urb
        elseif(aprof.eq.3) then
          write(iolst,*)'  Polynomial Factors: C0 = ',C0_Rrl,
     &   ' C1 = ',C1_Rrl,' C2 = ',C2_Rrl,' C3 = ',C3_Rrl,
     &   ' C4 = ',C4_Rrl,' C5 = ',C5_Rrl,' C6 = ',C6_Rrl
        elseif(aprof.eq.4) then
          write(iolst,*)'  Polynomial Factors: C0 = ',C0_Ind,
     &   ' C1 = ',C1_Ind,' C2 = ',C2_Ind,' C3 = ',C3_Ind,
     &   ' C4 = ',C4_Ind,' C5 = ',C5_Ind,' C6 = ',C6_Ind
        elseif(aprof.eq.5) then
          write(iolst,*)'  Polynomial Factors: C0 = ',C0_Cst,
     &   ' C1 = ',C1_Cst,' C2 = ',C2_Cst,' C3 = ',C3_Cst,
     &   ' C4 = ',C4_Cst,' C5 = ',C5_Cst,' C6 = ',C6_Cst
        elseif(aprof.eq.6) then
          write(iolst,*)'  Polynomial Factors: C0 = ',C0_EPA,
     &   ' C1 = ',C1_EPA,' C2 = ',C2_EPA,' C3 = ',C3_EPA,
     &   ' C4 = ',C4_EPA,' C5 = ',C5_EPA,' C6 = ',C6_EPA
        elseif(aprof.eq.7) then
          write(iolst,*)'  Exponential Profile: AFACT = ',afact
          write(iolst,*)'                     BFACT = ',bfact
        endif
      elseif(mode.eq.2) then
       write(iolst,*)'Ozone Limiting Method Selected:'
       write(iolst,*)
       write(iolst,*)'Section 3: OLM Options'
       write(iolst,*)
       write(iolst,*)'  OCOMP = ',ocomp
       write(iolst,*)'    1 = All sources compete for ambient ozone'
       write(iolst,*)'    2 = Each source sees full ambient ozone'
       write(iolst,*)
       write(iolst,*)'  Equilibrium ratio EQUIL = ',equil
       write(iolst,*)
       write(iolst,*)'  Background ozone OZSRC = ',ozsrc
       write(iolst,*)'     1 = hourly ozone.dat file'
       write(iolst,*)'     2 = monthly values'
       write(iolst,*)
       if (ozsrc.eq.1) then
         write(iolst,*)'  Hourly value read from: ',OZFILE
       elseif(ozsrc.eq.2) then
         write(iolst,*)'  12 Monthly values used: (',ozmonth,')'
       endif
       write(iolst,*)
      endif

c --- End it?
      if(ierr.GT.0)then
         stop 'HALTED in CONTROL -- see list file'
      endif

      return
      end

c-----------------------------------------------------------------------
      subroutine hd_cfg(iolst,ioinp,ioout,k,cversion,clevel,ierr)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403          HD_CFG
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Read first section of a CALPUFF output file header to
c               identify model configuration, and transfer corresponding
c               records to CALPUFF output arrays (with modifications)
c               Write comment records to header of output files
c
c --- UPDATES:
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOINP - integer - File unit number for input CALPUFF file
c          IOOUT - integer - File unit number for output CALPUFF file
c              k - integer - Sequential number for input CALPUFF file
c       CVERSION - char*12 - CALNO2 version
c         CLEVEL - char*12 - CALNO2 level (e.g., date)
c
c --- OUTPUT:
c           IERR - integer - Error flag (0 = No Error)
c
c --- HD_CFG called by: (main)
c --- HD_CFG calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use userinf
      use header

      implicit none

c --- Arguments
      integer :: iolst,ioinp,ioout,k,ierr
      character(len=12)   :: cversion,clevel

c --- Locals
      logical :: lout
      integer :: msource_out
      integer :: i,iok,idum
      integer :: kbyr,kbjul,kbhr,kbsec,krlg,ireport
      character(len=132)  :: comment, blank132

c --- Use variable names as listed in the CALPUFF Users Guide

c --- Initializations
      ierr=0
      idum=0
      do i=1,132
         blank132(i:i)=' '
      enddo
      msource_out=0

c --- Output is based on the first CALPUFF file read
      lout=.FALSE.
      if(k.EQ.1) then
         if(lbinout) lout=.TRUE.
      endif

c --- Record #1 - File Declaration -- 24 words
      read(ioinp)  dataset(k),dataver(k),datamod(k)
      if(lout) then 
        write(ioout) dataset(k),dataver(k),datamod(k)
cQC
c        write(iolst,*)dataset(k),dataver(k),datamod(k)
      endif

c --- QA dataset names
      iok=0
      iok=iok+INDEX(dataset(k),'CONC.DAT')

      if(iok.EQ.0) then
         write(iolst,*)'ERROR in HD_CFG -- Invalid dataset: '//
     &                  dataset//dataver
         write(iolst,*)'Requires Conc Dataset version 2.1 and later'
         ierr=1
         return
      endif

c --- Comment records
      read(ioinp)  ncom(k)

      if(lout) then
c ---    Add 2 new comment records to output
         write(ioout) (ncom(k)+2)
         comment='Processed by CALNO2 Version '//cversion//
     &           ' Level '//clevel//blank132(46:132)
         write(ioout) comment
         comment='NO2 from the set of CALPUFF output files is'//
     &           ' calculated'//blank132(55:132)
         write(ioout) comment
      endif

      do i=1,ncom(k)
         comment=blank132
         read(ioinp)  comment
         if(lout) write(ioout) comment
      enddo
c --- Control variables (record NCOM+3)

      if(dataver(k).EQ.'2.1             ') then
c ---------------------------------------------
         read(ioinp) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),xbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),
     &      meshdn(k),(nsbytype(i,k),i=1,8),
     &      msource(k),ndrec(k),nctrec(k),lsamp(k),
     &      nspout(k),lcompr(k),i2dmet(k),iutmzn(k),feast(k),fnorth(k),
     &      rnlat0(k),relon0(k),xlat1(k),xlat2(k),
     &      pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
         nsrctype(k)=8
c ---    Rest of source-types are zero;  8 in this file
         do i=nsrctype(k)+1,mxsrctype
            nsbytype(i,k)=0
         enddo
c ---    No receptor groups
         nrgrp(k)=0
c ---    No specie-level units in header
         lspunit(k)=.FALSE.
c ---    Translate real time zone to character (allow output to more recent file format)
         call UTC_ZONE(iolst,'REAL2CHR',axbtz(k),xbtz(k),idum,ierr)
         if(ierr.NE.0) stop 'ERROR in HD_CFG -- See list file'

      elseif(dataver(k).EQ.'2.2             ') then
c -------------------------------------------------
         read(ioinp) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),axbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),
     &      meshdn(k),(nsbytype(i,k),i=1,8),
     &      msource(k),ndrec(k),nctrec(k),lsamp(k),
     &      nspout(k),lcompr(k),i2dmet(k),iutmzn(k),feast(k),fnorth(k),
     &      rnlat0(k),relon0(k),xlat1(k),xlat2(k),
     &      pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
         nsrctype(k)=8
c ---    Rest of source-types are zero;  8 in this file
         do i=nsrctype(k)+1,mxsrctype
            nsbytype(i,k)=0
         enddo
c ---    No receptor groups
         nrgrp(k)=0
c ---    Specie-level units are in header
         lspunit(k)=.TRUE.
         if(ierr.NE.0) stop 'ERROR in HD_CFG -- See list file'

      elseif(dataver(k).EQ.'7.0             ' .OR.
     &       dataver(k).EQ.'TNG-3.0         ') then
c ---------------------------------------------
         read(ioinp) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),axbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),
     &      meshdn(k),nsrctype(k),msource(k),
     &      ndrec(k),nrgrp(k),nctrec(k),lsamp(k),
     &      nspout(k),lcompr(k),i2dmet(k),iutmzn(k),feast(k),fnorth(k),
     &      rnlat0(k),relon0(k),xlat1(k),xlat2(k),
     &      pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
c ---    Number of each type of source in next record (record NCOM+3a)
         read(ioinp) (nsbytype(i,k),i=1,nsrctype(k))
c ---    Rest of source-types are zero
         do i=nsrctype(k)+1,mxsrctype
            nsbytype(i,k)=0
         enddo
c ---    Specie-level units are in header
         lspunit=.TRUE.
         if(ierr.NE.0) stop 'ERROR in HD_CFG -- See list file'

      else
         write(iolst,*)'ERROR in HD_CFG -- Invalid dataset: '//
     &                  dataset//dataver
         write(iolst,*)'Requires Dataset version 2.1 and later format'
         ierr=1
      endif

c --- Derived data
      nsrc(k)=0
      do i=1,mxsrctype
         nsrc(k)=nsrc(k)+nsbytype(i,k)
      enddo
      if(lsamp(k)) then
         ngx(k)=meshdn(k)*(iesamp(k)-ibsamp(k))+1
         ngy(k)=meshdn(k)*(jesamp(k)-jbsamp(k))+1
      else
         ngx(k)=0
         ngy(k)=0
      endif
      ldisc(k)=.FALSE.
      if(ndrec(k).GT.0) ldisc(k)=.TRUE.
      lctsg(k)=.FALSE.
      if(nctrec(k).GT.0) lctsg(k)=.TRUE.

      if(lout) then
c ---    Place values into output header array slot
         dataver(nfiles+1) = dataver(k)
         cmodel(nfiles+1)  = cmodel(k)
         ver(nfiles+1)     = ver(k)  
         level(nfiles+1)   = level(k)
         ibyr(nfiles+1)    = ibyr(k)
         ibjul(nfiles+1)   = ibjul(k)
         ibhr(nfiles+1)    = ibhr(k)
         ibsec(nfiles+1)   = ibsec(k)
         xbtz(nfiles+1)    = xbtz(k)
         axbtz(nfiles+1)   = axbtz(k)
         irlg(nfiles+1)    = irlg(k)
         iavg(nfiles+1)    = iavg(k)
         nsecdt(nfiles+1)  = nsecdt(k)
         nxm(nfiles+1)     = nxm(k)
         nym(nfiles+1)     = nym(k)
         dxkm(nfiles+1)    = dxkm(k)
         dykm(nfiles+1)    = dykm(k)
         nz(nfiles+1)      = nz(k)
         xorigkm(nfiles+1) = xorigkm(k)
         yorigkm(nfiles+1) = yorigkm(k)
         nssta(nfiles+1)   = nssta(k)
         ibcomp(nfiles+1)  = ibcomp(k)
         iecomp(nfiles+1)  = iecomp(k)
         jbcomp(nfiles+1)  = jbcomp(k)
         jecomp(nfiles+1)  = jecomp(k)
         ibsamp(nfiles+1)  = ibsamp(k)
         jbsamp(nfiles+1)  = jbsamp(k)
         iesamp(nfiles+1)  = iesamp(k)
         jesamp(nfiles+1)  = jesamp(k)
         meshdn(nfiles+1)  = meshdn(k)
         msource(nfiles+1) = msource_out
         nsrctype(nfiles+1)= nsrctype(k)
         ndrec(nfiles+1)   = ndrec(k)
         nrgrp(nfiles+1)   = nrgrp(k)
         nctrec(nfiles+1)  = nctrec(k)
         lsamp(nfiles+1)   = lsamp(k)
         nspout(nfiles+1)  = nspout(k)
         lcompr(nfiles+1)  = lcompr(k)
         i2dmet(nfiles+1)  = i2dmet(k)
         iutmzn(nfiles+1)  = iutmzn(k)
         feast(nfiles+1)   = feast(k)
         fnorth(nfiles+1)  = fnorth(k)
         rnlat0(nfiles+1)  = rnlat0(k)
         relon0(nfiles+1)  = relon0(k)
         xlat1(nfiles+1)   = xlat1(k)
         xlat2(nfiles+1)   = xlat2(k)
         pmap(nfiles+1)    = pmap(k)
         utmhem(nfiles+1)  = utmhem(k)
         datum(nfiles+1)   = datum(k)
         daten(nfiles+1)   = daten(k)
         clat0(nfiles+1)   = clat0(k)
         clon0(nfiles+1)   = clon0(k)
         clat1(nfiles+1)   = clat1(k)
         clat2(nfiles+1)   = clat2(k)
         lspunit(nfiles+1) = lspunit(k)
c ---    Derived data
         ngx(nfiles+1)     = ngx(k)
         ngy(nfiles+1)     = ngy(k)
         ldisc(nfiles+1)   = ldisc(k)
         lctsg(nfiles+1)   = lctsg(k)
      endif

c ---    Calculate total source counts across all files
      do i=1,mxsrctype
         nsbytype(i,nfiles+1)= nsbytype(i,nfiles+1)+nsbytype(i,k)
      enddo
      nsrc(nfiles+1)    = nsrc(k) + nsrc(nfiles+1)

      return
      end

c-----------------------------------------------------------------------
      subroutine hd_proc(ioinp,ioout,iolst,k)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403           HD_PROC
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Read remainder of a CALPUFF output file header and 
c               transfer records to output file arrays
c
c --- UPDATES:
c
c --- INPUT:
c          IOINP - integer - File unit number for input CALPUFF file
c          IOOUT - integer - File unit number for output CALPUFF file
c              k - integer - Sequential number for output CALPUFF file
c
c --- OUTPUT:
c
c --- HD_PROC called by: (main)
c --- HD_PROC calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header
      use userinf

      implicit none

c --- Arguments
      integer :: ioinp,ioout,k,iolst

c --- Locals
      character(len=16) :: units
      integer :: i,j,n,itype,m

c --- Title (record NCOM+4)
      read(ioinp)  (title(i,k),i=1,3)

c --- Specie/level list (record NCOM+5)
      read(ioinp)  (csout(i,k),i=1,nspout(k))

c --- Units for each specie/level (record NCOM+5a)
      if(lspunit(k)) then
         read(ioinp)  (csunit(i,k),i=1,nspout(k))
      else
c ---    Set units based on dataset
         units='g/m3            '
         do i=1,nspout(k)
            csunit(i,k)=units
         enddo
      endif

c --- Non-gridded receptor coordinates (record NCOM+6)
c --- (New in dataset 7.0 / TNG-3.0: htrec, irgrp)
      if(ndrec(k).NE.0) then
         if(nrgrp(k).EQ.0) then
            read(ioinp) (xrec(i,k),i=1,ndrec(k)),
     &                  (yrec(i,k),i=1,ndrec(k)),
     &                  (zrec(i,k),i=1,ndrec(k))

         elseif(nrgrp(k).GE.1) then
c ---       (7.0 / TNG-3.0 and up record NCOM+6, 6a)
            read(ioinp) (xrec(i,k),i=1,ndrec(k)),
     &                  (yrec(i,k),i=1,ndrec(k)),
     &                  (zrec(i,k),i=1,ndrec(k)),
     &                  (htrec(i,k),i=1,ndrec(k)),
     &                  (irgrp(i,k),i=1,ndrec(k))
            read(ioinp) (argrpnam(n,k),n=1,nrgrp(k))
         endif
      endif

c --- Complex terrain receptor coordinates (record NCOM+7)
      if(nctrec(k).NE.0) then
         read(ioinp)  (xctr(i,k),i=1,nctrec(k)),
     &                (yctr(i,k),i=1,nctrec(k)),
     &                (zctr(i,k),i=1,nctrec(k)),
     &                (ihill(i,k),i=1,nctrec(k))
      endif

c --- Source names (records NCOM+8 to NCOM+15)
c --- Loop over source types
      do i=1,mxsrctype
         if(nsbytype(i,k).GT.0) then
            read(ioinp)  itype,(csrcnam(n,i,k),n=1,nsbytype(i,k))
            if(itype.NE.i) then
c ---          Report problem information and quit
               write(*,*)
               write(*,*)
               write(*,*)'  *  Unexpected source type found, or   *'
               write(*,*)'  *  Unexpected source number found.    *'
               write(*,*)'  *  Source type expected: ',i
               write(*,*)'  *  Source type found   : ',itype
               write(*,*)'  *  Source Names Read:                 *'
               write(*,*) (csrcnam(n,i,k),n=1,nsbytype(i,k))
               stop
            endif
         endif
      enddo

c --- Output is based on the first CALPUFF file read
c --------------------------------------------------
      if(k.NE.1) return

      j=nfiles+1

c --- Title (record NCOM+4)
      do i=1,3
         title(i,j)=title(i,k)
      enddo

c --- Specie/level list (record NCOM+5)
      do i=1,nspout(k)
         if(csout(i,k).eq.'NOX           1') then
           csout(i,j)='NO2           1'
         else
           csout(i,j)=csout(i,k)
         endif
      enddo

c --- Units for each specie/level (record NCOM+5a)
      do i=1,nspout(k)
         csunit(i,j)=csunit(i,k)
      enddo

c --- Non-gridded receptor coordinates (record NCOM+6)
      if(ndrec(k).NE.0) then
         if(nrgrp(k).EQ.0) then
            do i=1,ndrec(k)
               xrec(i,j)=xrec(i,k)
               yrec(i,j)=yrec(i,k)
               zrec(i,j)=zrec(i,k)
            enddo
         elseif(nrgrp(k).GE.1) then
c ---       (7.0 / TNG-3.0 and up record NCOM+6, 6a)
            do i=1,ndrec(k)
               xrec(i,j)=xrec(i,k)
               yrec(i,j)=yrec(i,k)
               zrec(i,j)=zrec(i,k)
               htrec(i,j)=htrec(i,k)
               irgrp(i,j)=irgrp(i,k)
            enddo
            do i=1,nrgrp(k)
               argrpnam(i,j)=argrpnam(i,k)
            enddo
         endif
      endif

c --- Complex terrain receptor coordinates (record NCOM+7)
      if(nctrec(k).NE.0) then
         do i=1,nctrec(k)
            xctr(i,j)=xctr(i,k)
            yctr(i,j)=yctr(i,k)
            zctr(i,j)=zctr(i,k)
            ihill(i,j)=ihill(i,k)
         enddo
      endif

      return
      end

c-----------------------------------------------------------------------
      subroutine hd_out(ioout,k)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403            HD_OUT
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Write remainder of output file CALPUFF header after
c               comment record section
c
c --- INPUT:
c          IOOUT - integer - File unit number for output CALPUFF file
c              k - integer - Array index for stored data
c
c --- OUTPUT: NONE
c
c --- HD_OUT called by: (main)
c --- HD_OUT calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header

      implicit none

c --- Declare arguments
      integer :: ioout,k

c --- Locals
      logical :: lout
      integer :: i,n,itype,nout,l


c --- Assign Output Source names (records NCOM+8 ...)
c --- Loop over source types
      do i=1,mxsrctype
       nout=1
       do l=1,k-1
         if(nsbytype(i,l).GT.0) then
            do n=1,nsbytype(i,l)
               csrcnam(nout,i,k)=csrcnam(n,i,l)
               nout=nout+1
            enddo
         endif
       enddo
      enddo


c --- Control variables (record NCOM+3)
      if(dataver(k).EQ.'2.1             ') then
         write(ioout) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),xbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),meshdn(k),
     &      (nsbytype(i,k),i=1,8),msource(k),ndrec(k),nctrec(k),
     &      lsamp(k),nspout(k),lcompr(k),i2dmet(k),iutmzn(k),
     &      feast(k),fnorth(k),rnlat0(k),relon0(k),
     &      xlat1(k),xlat2(k),pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
      elseif(dataver(k).EQ.'2.2             ') then
         write(ioout) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),axbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),meshdn(k),
     &      (nsbytype(i,k),i=1,8),msource(k),ndrec(k),nctrec(k),
     &      lsamp(k),nspout(k),lcompr(k),i2dmet(k),iutmzn(k),
     &      feast(k),fnorth(k),rnlat0(k),relon0(k),
     &      xlat1(k),xlat2(k),pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
      elseif(dataver(k).EQ.'7.0             ' .OR.
     &       dataver(k).EQ.'TNG-3.0         ') then
         write(ioout) cmodel(k),ver(k),level(k),ibyr(k),ibjul(k),
     &      ibhr(k),ibsec(k),axbtz(k),irlg(k),iavg(k),nsecdt(k),nxm(k),
     &      nym(k),dxkm(k),dykm(k),nz(k),xorigkm(k),yorigkm(k),
     &      nssta(k),ibcomp(k),iecomp(k),jbcomp(k),jecomp(k),
     &      ibsamp(k),jbsamp(k),iesamp(k),jesamp(k),meshdn(k),
     &      nsrctype(k),msource(k),ndrec(k),nrgrp(k),nctrec(k),
     &      lsamp(k),nspout(k),lcompr(k),i2dmet(k),iutmzn(k),
     &      feast(k),fnorth(k),rnlat0(k),relon0(k),
     &      xlat1(k),xlat2(k),pmap(k),utmhem(k),datum(k),daten(k),
     &      clat0(k),clon0(k),clat1(k),clat2(k)
c ---    Control variables (record NCOM+3a)
         write(ioout) (nsbytype(i,k),i=1,nsrctype(k))


      endif

c --- Title (record NCOM+4)
      write(ioout) (title(i,k),i=1,3)

c --- Specie/level list (record NCOM+5)
      write(ioout) (csout(i,k),i=1,nspout(k))

c --- Units for each specie/level (record NCOM+5a)
      if(lspunit(k)) then
         write(ioout) (csunit(i,k),i=1,nspout(k))
      endif

c --- Non-gridded receptor coordinates (record NCOM+6)
c --- (New in dataset 7.0 / TNG-3.0: htrec, irgrp)
      if(ndrec(k).NE.0) then
         if(nrgrp(k).EQ.0) then
            write(ioout)(xrec(i,k),i=1,ndrec(k)),
     &                  (yrec(i,k),i=1,ndrec(k)),
     &                  (zrec(i,k),i=1,ndrec(k))
         elseif(nrgrp(k).GE.1) then
c ---       (7.0 / TNG-3.0 and up record NCOM+6, 6a)
            write(ioout)(xrec(i,k),i=1,ndrec(k)),
     &                  (yrec(i,k),i=1,ndrec(k)),
     &                  (zrec(i,k),i=1,ndrec(k)),
     &                  (htrec(i,k),i=1,ndrec(k)),
     &                  (irgrp(i,k),i=1,ndrec(k))
            write(ioout)(argrpnam(n,k),n=1,nrgrp(k))
         endif
      endif

c --- Complex terrain receptor coordinates (record NCOM+7)
      if(nctrec(k).NE.0) then
         write(ioout) (xctr(i,k),i=1,nctrec(k)),
     &                (yctr(i,k),i=1,nctrec(k)),
     &                (zctr(i,k),i=1,nctrec(k)),
     &                (ihill(i,k),i=1,nctrec(k))
      endif

c --- Source names (records NCOM+8 ...)
c --- Loop over source types
      do i=1,mxsrctype
         if(nsbytype(i,k).GT.0) then
            write(ioout) i,(csrcnam(n,i,k),n=1,nsbytype(i,k))
         endif
      enddo

      return
      end
c-----------------------------------------------------------------------
      subroutine alloc_1(k)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403           ALLOC_1
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Allocate arrays of header information that depend only
c               on the number of CALPUFF files processed
c
c --- INPUT:
c              k - integer - Number of CALPUFF files
c
c --- OUTPUT:
c
c --- ALLOC_1 called by: (main)
c --- ALLOC_1 calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header

      implicit none

c --- Arguments
      integer :: k

c --- Logical arrays (read from file)
      allocate (lcompr(k),lsamp(k))

c --- Logical arrays (derived)
      allocate (ldisc(k),lctsg(k),lspunit(k))

c --- Character arrays (read from file)
      allocate (cmodel(k),ver(k),level(k))
      allocate (dataset(k),dataver(k),datamod(k))
      allocate (title(3,k))
      allocate (pmap(k),datum(k),axbtz(k),daten(k),utmhem(k))
      allocate (clat0(k),clon0(k),clat1(k),clat2(k))

c --- Integer arrays (read from file)
      allocate (ncom(k),ibyr(k),ibjul(k),ibhr(k),ibsec(k),irlg(k))
      allocate (iavg(k),nsecdt(k),nxm(k),nym(k),nz(k),nssta(k))
      allocate (ibcomp(k),iecomp(k),jbcomp(k),jecomp(k))
      allocate (ibsamp(k),jbsamp(k),iesamp(k),jesamp(k))
      allocate (meshdn(k),nsrctype(k),msource(k))
      allocate (ndrec(k),nrgrp(k),nctrec(k),nspout(k))
      allocate (i2dmet(k),iutmzn(k))
      allocate (nsbytype(mxsrctype,k))

c --- Integer arrays (derived)
      allocate (nsrc(k),ngx(k),ngy(k))

c --- Real arrays (read from file)
      allocate (xbtz(k),dxkm(k),dykm(k),xorigkm(k),yorigkm(k))
      allocate (feast(k),fnorth(k))
      allocate (rnlat0(k),relon0(k),xlat1(k),xlat2(k))

      return
      end

c-----------------------------------------------------------------------
      subroutine hd_qa(iolst)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403             HD_QA
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Compare selected file header data that should be the
c               same in all files
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c
c --- OUTPUT:
c
c --- HD_QA called by: (main)
c --- HD_QA calls:     DATE_HRSEC
c----------------------------------------------------------------------
c --- Access module data
      use header
      use userinf

      implicit none

c --- Declare arguments
      integer :: iolst

c --- Locals
      integer (kind=8) :: start1,startk
      integer :: i,k,nwarn,nfatal,ierr,no

c --- Initializations
      ierr=0
      nwarn=0
      nfatal=0

c --- Notice to list file
      write(iolst,*)
      write(iolst,*)'HD_QA:  Comparing selected header information'
      write(iolst,*)'---------------------------------------------'
      write(iolst,*)

c --- Selected operations on the first file information
      call DATE_HRSEC(iolst,'TOSNGL',ibyr(1),ibjul(1),ibhr(1),ibsec(1),
     &                start1,ierr)
      if(ierr.NE.0) stop 'HALTED in HD_QA -- See list file'

c --- Compare each file header with the first
      do k=2,nfiles
         write(iolst,*) '          File 1 Compared to File ',k

c ---    Dataset type
         if(dataset(1).NE.dataset(k)) then
            nfatal=nfatal+1
            write(iolst,*)'Type of data in file ---'
            write(iolst,*)'  FATAL: dataset ',dataset(1),dataset(k)
         endif

c ---    Times
         call DATE_HRSEC(iolst,'TOSNGL',ibyr(k),ibjul(k),ibhr(k),
     &                   ibsec(k),startk,ierr)
         if(ierr.NE.0) stop 'HALTED in HD_QA -- See list file'

         if(startk.NE.start1 .OR.
     &      irlg(1).NE.irlg(k)) then
            nwarn=nwarn+1
            write(iolst,*)'Times ---'
            write(iolst,*)'WARNING: start  ',start1,startk
            write(iolst,*)'         irlg   ',irlg(1),irlg(k)
         endif
         if(axbtz(1).NE.axbtz(k) .OR.
     &      iavg(1).NE.iavg(k)   .OR.
     &      nsecdt(1).NE.nsecdt(k)) then
            nfatal=nfatal+1
            write(iolst,*)'Times ---'
            write(iolst,*)'  FATAL: axbtz  ',axbtz(1),axbtz(k)
            write(iolst,*)'         iavg   ',iavg(1),iavg(k)
            write(iolst,*)'         nsecdt ',nsecdt(1),nsecdt(k)
         endif

c ---    Receptors

         if(lsamp(1).NEQV.lsamp(k) .OR.
     &      ndrec(1).NE.ndrec(k) .OR.
     &      nctrec(1).NE.nctrec(k)) then
            nfatal=nfatal+1
            write(iolst,*)'Receptor types ---'
            write(iolst,*)'  FATAL: lsamp  ',lsamp(1),lsamp(k)
            write(iolst,*)'         ndrec  ',ndrec(1),ndrec(k)
            write(iolst,*)'         nctrec ',nctrec(1),nctrec(k)
         else
            if(lsamp(1)) then
               if(nxm(1).NE.nxm(k) .OR. nym(1).NE.nym(k) .OR.
     &            dxkm(1).NE.dxkm(k) .OR. dykm(1).NE.dykm(k) .OR.
     &            xorigkm(1).NE.xorigkm(k) .OR.
     &            yorigkm(1).NE.yorigkm(k) .OR.
     &            ibsamp(1).NE.ibsamp(k) .OR.
     &            iesamp(1).NE.iesamp(k) .OR.
     &            jbsamp(1).NE.jbsamp(k) .OR.
     &            jesamp(1).NE.jesamp(k) .OR.
     &            meshdn(1).NE.meshdn(k)) then
                  nfatal=nfatal+1
            write(iolst,*)'Gridded Receptors ---'
            write(iolst,*)'  FATAL: nx     ',nxm(1),nxm(k)
            write(iolst,*)'         ny     ',nym(1),nym(k)
            write(iolst,*)'         dx     ',dxkm(1),dxkm(k)
            write(iolst,*)'         dy     ',dykm(1),dykm(k)
            write(iolst,*)'         xorig  ',xorigkm(1),xorigkm(k)
            write(iolst,*)'         yorig  ',yorigkm(1),yorigkm(k)
            write(iolst,*)'         ibsamp ',ibsamp(k),ibsamp(k)
            write(iolst,*)'         iesamp ',iesamp(k),iesamp(k)
            write(iolst,*)'         jbsamp ',jbsamp(k),jbsamp(k)
            write(iolst,*)'         jesamp ',jesamp(k),jesamp(k)
            write(iolst,*)'         meshdn ',meshdn(1),meshdn(k)
               endif
            endif

            if(ldisc(1)) then
               no=0
               do i=1,ndrec(1)
                  if(xrec(i,1).NE.xrec(i,k) .OR. 
     &               yrec(i,1).NE.yrec(i,k)) then
                    no=no+1
                   endif
               enddo
               if(no.NE.0) then
                  nfatal=nfatal+1
            write(iolst,*)'Discrete Receptors ---'
            write(iolst,*)'  FATAL: number that do not match = ',no
               endif
            endif

            if(lctsg(1)) then
               no=0
               do i=1,nctrec(1)
                  if(xctr(i,1).NE.xctr(i,k) .OR. 
     &               yctr(i,1).NE.yctr(i,k)) no=no+1
               enddo
               if(no.NE.0) then
                  nfatal=nfatal+1
            write(iolst,*)'CTSG Receptors ---'
            write(iolst,*)'  FATAL: number that do not match = ',no
               endif
            endif
         endif

c ---    Species
         if(nspout(1).NE.nspout(k) .OR.
     &      lspunit(1).NEQV.lspunit(k)) then
            write(iolst,*)'Species ---'
            write(iolst,*)'  FATAL: nspout ',nspout(1),nspout(k)
            write(iolst,*)'        lspunit ',lspunit(1),lspunit(k)
         else
            no=0
            do i=1,nspout(1)
               if(csout(i,1).NE.csout(i,k)) no=no+1
            enddo
            if(no.NE.0) then
            write(iolst,*)'Species Output ---'
            write(iolst,*)'  FATAL: number that do not match = ',no
               do i=1,nspout(1)
            write(iolst,*)'        csout   ',csout(i,1),csout(i,k)
               enddo
            endif
         endif
      enddo

      if(nwarn.GT.0) then
         write(*,*)'HD_QA:  File Header WARNINGS -- See list file'
      endif
      if(nfatal.GT.0) then
         stop 'HALTED in HD_QA -- See list file'
      endif

      return
      end
c-----------------------------------------------------------------------
      subroutine compolm(iolst,ioout,ioz,ierr)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403            COMPOLM
c               C. DesAutels, Exponent Inc.
c
c --- PURPOSE:  Process data records to perform OLM calculations
c
c --- UPDATES:
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOPER - integer - File unit no. for output Time-Pd Max file
c          IOOUT - integer - File unit number for output CALPUFF file
c          IOOZ  - integer - File unit number for hourly ozone file
c
c --- OUTPUT:
c           IERR - integer - Error flag (0 = No Error)
c
c --- COMPOLM called by: (main)
c --- COMPOLM calls:     GETPER, PUTPER, OLMCALC1, OLMCALC2, OZHD, OZRD
c----------------------------------------------------------------------
c --- Access module data
      use userinf
      use header
      use inpdata

      implicit none

c --- Declare arguments
      integer :: iolst,ioinp,ioout,ierr,ioz

c --- Locals
      integer :: k, ieof, izero
      integer :: iyr,jul,ihr,isec
      integer (kind=8) :: ibegper,iendper

      real :: ozone

c --- Initializations
      ierr=0
      ieof=0
      izero=0

c --- open and read background ozone file header (time invarient) records
      if(ozsrc.eq.1) then
       open(ioz,file=ozfile,status='old')
       call ozhd(iolst,ioz)
      endif

10    continue
        
      do while (ieof.EQ.0)
        
       do k=1,nfiles
c --- Read 1 Time Period of Concentration Data
         call GETPER(iolst,ioinparray(k),k,ierr,ieof)
       enddo

c --- Read and save the matching record of ozone data

       if(ieof.eq.0) then
        if(ozsrc.eq.1) then
          call ozrd(iolst,ioz,ozone)
        elseif(ozsrc.eq.2) then
          call ozmon(iolst,ozone)
        else
          write(*,*)'Invalid value OZSRC'
        endif 

        if(ocomp.eq.1) then 
         call OLMCALC1(ozone,iolst)
        elseif(ocomp.eq.2) then
         call OLMCALC2(ozone,iolst)
        else
         stop 'INVALID VALUE FOR OCOMP'
        endif
        call PUTPER(iolst,ioout,ierr)
       endif

      enddo

      end

c-----------------------------------------------------------------------
      subroutine comparm(iolst,ioout,ierr)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403           COMPARM
c               C. DesAutels, Exponent Inc.
c
c --- PURPOSE:  Process data records to perform ARM calculations

c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOOUT - integer - File unit number for output CALPUFF file
c
c --- OUTPUT:
c           IERR - integer - Error flag (0 = No Error)
c
c --- COMPARM called by: (main)
c --- COMPARM calls:     GETPER, PUTPER, ARMCALC
c----------------------------------------------------------------------
c --- Access module data
      use userinf
      use header
      use inpdata

      implicit none

c --- Declare arguments
      integer :: iolst,ioinp,ioout,ierr,ioz

c --- Locals
      integer :: k, ieof, izero
      integer :: iyr,jul,ihr,isec
      integer (kind=8) :: ibegper,iendper

c --- Initializations
      ierr=0
      ieof=0
      izero=0

      do while (ieof.EQ.0)
        
       do k=1,nfiles
c --- Read 1 Time Period of Concentration Data
         call GETPER(iolst,ioinparray(k),k,ierr,ieof)
       enddo

       if(ieof.eq.0) then
        call ARMCALC(iolst)
        call PUTPER(iolst,ioout,ierr)
       endif

      enddo
 
      end

c-----------------------------------------------------------------------
      subroutine getper(iolst,ioinp,k,ierr,ieof)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403            GETPER
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Read and store model data records for 1 time period
c
c               Receptor arrays must be dimensioned EXACTLY for the
c               number(s) of receptors in the CALPUFF file!
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOINP - integer - File unit number for input CALPUFF file
c              K - integer - Current CALPUFF file index
c
c --- OUTPUT:
c           IERR - integer - Error flag (0 = No Error)
c           IEOF - integer - End of file flag
c
c --- GETPER called by: COMPOLM, COMPARM
c --- GETPER calls:     DATE_HRSEC, CMPRSIO, RDBDAT
c----------------------------------------------------------------------
c --- Access module data
      use header
      use inpdata

      implicit none

c --- Declare arguments
      integer :: iolst,ioinp,k,ierr,ieof

c --- Locals
      integer :: is,nsources,isp
      integer :: nxy,ncmprs,ione

c --- Initializations
      ione=0
      ieof=0
      ierr=0

c --- Source contribution configuration
        if(msource(k).EQ.0) then
c ---    Sum of all sources
         if(nsrc(k).GT.1) then
            write(*,*)'Error in INPUTS: for OLM, CALPUFF must be run '//
     &            'with MSOURCE, or CONC file must have only 1 source'
            stop
         endif 
         nsources=1
        else
c ---    Individual sources plus total
         nsources=nsrc(k)+1
        endif

c --- Number of gridded receptors
      nxy=ngx(k)*ngy(k)

c --- Source loop
c ---------------
      do is=1,nsources

c ---    Record 1:  Time period
         read(ioinp,end=999)
     &        nyrb(k),njulb(k),nhrb(k),nsecb(k),
     &        nyre(k),njule(k),nhre(k),nsece(k)


c ---    Record 2:  Source information
         read(ioinp,end=999)
     &        istype(is,k),isnum(is,k),sname(is,k),
     &        sxkm(is,k),sykm(is,k)

c ---    Output species-level loop
c --------------------------------
         do isp=1,nspout(k)
            if(lsamp(k)) then
               if(lcompr(k)) then
                  read(ioinp) ncmprs
                  call CMPRSIO(iolst,ioinp,'GETR',
     &                         datag(1,1,isp,is,k),nxy,
     &                         workg,ncmprs,cspecg(isp,k),ierr)

                  if(ierr.NE.0) stop 'HALTED in GETPER -- See list file'
               else
                  call RDBDAT(ioinp,cspecg(isp,k),
     &                        datag(1,1,isp,is,k),ngx(k),ngy(k))
               endif
            endif

            if(ldisc(k)) then
               if(lcompr(k))then
                  read(ioinp) ncmprs

                  call CMPRSIO(iolst,ioinp,'GETR',
     &                         datad(1,isp,is,k),ndrec(k),
     &                         workd,ncmprs,cspecd(isp,k),ierr)
                  if(ierr.NE.0) stop 'HALTED in GETPER -- See list file'
               else
                  call RDBDAT(ioinp,cspecd(isp,k),
     &                        datad(1,isp,is,k),ndrec(k),ione)
               endif
            endif

            if(lctsg(k)) then
               if(lcompr(k))then
                  read(ioinp) ncmprs
                  call CMPRSIO(iolst,ioinp,'GETR',
     &                         datact(1,isp,is,k),nctrec(k),
     &                         workct,ncmprs,cspecct(isp,k),ierr)
                  if(ierr.NE.0) stop 'HALTED in GETPER -- See list file'
               else
                  call RDBDAT(ioinp,cspecct(isp,k),
     &                        datact(1,isp,is,k),nctrec(k),ione)
               endif
            endif
         enddo
c --------------------------------
c ---    End output species-level loop
      enddo
c -------------------
c --- End source loop

      return

999   write(*,*)'Reached end of CALPUFF file ',k
      ieof=1
      return

      end
c-----------------------------------------------------------------------
      subroutine ozhd(iolst,ioz)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403            OZHD
c               C. DesAutels, Exponent Inc.
c
c --- PURPOSE:  Read header of OZONE.DAT file and perform QA processes 
c               for OLM processing 
c
c --- UPDATES:
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOZ   - integer - File unit number for input OZONE file
c
c --- OUTPUT:
c
c --- OZHD   called by: COMPOLM
c --- OZHD   calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use userinf
      use header

      implicit none

c --- Arguments
      integer :: ioz,iolst

c --- Locals
      integer :: ozbyr,ozbjday,ozbhr,ozbsec
      integer :: ozeyr,ozejday,ozehr,ozesec
      integer :: ozbegin,calbegin
      integer :: nozsta,icom,i
      character(len=8)  :: ozxbtz,prj


c --- Read header records.  Store time zone and time ranges for QA. Skip others.
c --- Limit applied that only the 1st ozone station in the file will be used
c --- if multiple stations are present.
      read(ioz,*)
      read(ioz,*)icom
      do i=1,icom
        read(ioz,*)
      enddo
      read(ioz,*)prj
      if(prj.eq.'UTM     ') then
        read(ioz,*)
      elseif(prj.eq.'LCC     ') then
        read(ioz,*)
        read(ioz,*)
      elseif(prj.eq.'TTM     ') then
        read(ioz,*)
        read(ioz,*)
      elseif(prj.eq.'PS      ') then
        read(ioz,*)
      elseif(prj.eq.'EM      ') then
        read(ioz,*)
      elseif(prj.eq.'LAZA    ') then
        read(ioz,*)
        read(ioz,*)
      else
        write(iolst,*)'Unknown Ozone Projection',prj
      endif
      read(ioz,*)
      read(ioz,*)
      read(ioz,'(a8)')ozxbtz
      if (ozxbtz.ne.axbtz(1)) then
         write(iolst,*)
         write(iolst,*)'FATAL: Time zone in Ozone file must '//
     &                       'match value in CALPUFF output'
         write(iolst,*)ozxbtz,axbtz(1)
         stop
      endif
      read(ioz,*)ozbyr,ozbjday,ozbhr,ozbsec,ozeyr,ozejday,ozehr,ozesec

c --- Check that Ozone data starts at or before the start of the CALPUFF output.
      ozbegin=ozbyr*1000000000+ozbjday*1000000+ozbhr*10000+ozbsec
      calbegin=ibyr(1)*1000000000+ibjul(1)*1000000+ibhr(1)*10000
     &                        +ibsec(1)
      if(ozbegin.gt.calbegin) then
         write(iolst,*)
         write(iolst,*)'FATAL: Ozone data begins after '//
     &                       'start of CALPUFF file'
         write(iolst,*)'OZONE ',ozbyr,' ',ozbjday,' ',ozbhr,' ',ozbsec
         write(iolst,*)'CALPUFF.DAT: ',ibyr(1),' ',ibjul(1),' ',
     &                     ibhr(1),' ',ibsec(1)
         stop
      endif

      read(ioz,*)nozsta
      if (nozsta.gt.1) then
         write(iolst,*)
         write(iolst,*)'WARNING: More than 1 ozone station in '//
     &                       'provided file'
         write(iolst,*)'Only the first ozone station will be  '//
     &                       'used in processing' 
      endif
      do i=1,nozsta
        read(ioz,*)
      enddo

      end
c-----------------------------------------------------------------------
      subroutine ozrd(iolst,ioz,ozoneppb)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403            OZRD
c               C. DesAutels, Exponent Inc.
c
c --- PURPOSE:  Read individual record of OZONE.DAT file for OLM processing 
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOZ   - integer - File unit number for input OZONE file
c
c --- OUTPUT:
c
c --- OZRD   called by: COMPOLM
c --- OZRD   calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use userinf
      use header
      use inpdata

      implicit none

c --- Arguments
      integer :: ioz,iolst

c --- Locals
      integer :: ozbyr,ozbjday,ozbhr,ozbsec
      integer :: ozeyr,ozejday,ozehr,ozesec
      integer(kind=8) :: oze,ozb,calmid
      integer :: nyrm,njulm,nhrm,nsecm
      integer :: nozsta
      integer :: ierr
      character(len=8)  :: ozxbtz
      real :: ozoneppb
      integer dtm,ieof

      ieof=0

50    read(ioz,*,iostat=ieof)ozbyr,ozbjday,ozbhr,ozbsec,
     &           ozeyr,ozejday,ozehr,ozesec,ozoneppb

      if(ieof.ne.0) then
       write(*,*)'Insufficient ozone data available in ozone.dat file'
       stop
      endif  

c --- For the first CALPUFF file read, Determine the midpoint of CALPUFF timestep  
c --- and if that midpoint is within the provided ozone record
       dtm=iavg(1)*nsecdt(1)/2
       nyrm=nyrb(1)
       njulm=njulb(1)
       nhrm=nhrb(1)
       nsecm=nsecb(1)

       call time_incr(iolst,nyrm,njulm,nhrm,nsecm,0,dtm,
     &                     ierr)

       if(ozbsec.eq.3600) then
         call time_incr(iolst,ozbyr,ozbjday,ozbhr,0,1,0,ierr)
       endif
       if(ozesec.eq.3600) then
         call time_incr(iolst,ozeyr,ozejday,ozehr,0,1,0,ierr)
       endif
        
       call date_hrsec(iolst,'TOSNGL',ozbyr,ozbjday,ozbhr,0,
     &                      ozb,ierr)
       call date_hrsec(iolst,'TOSNGL',ozeyr,ozejday,ozehr,0,
     &                      oze,ierr)
       call date_hrsec(iolst,'TOSNGL',nyrm,njulm,nhrm,nsecm,
     &                      calmid,ierr)
       if (calmid.gt.oze) then
          goto 50
       else
          continue
       endif
      end 
c-----------------------------------------------------------------------
      subroutine ozmon(iolst,ozone)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403            OZMON
c               C. DesAutels, Exponent Inc.
c
c --- PURPOSE:  Determine monthly value of ozone from INPUT file array
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c
c --- OUTPUT:
c           IERR - integer - Error flag (0 = No Error)
c
c --- OZMON  called by: COMPOLM
c --- OZMON  calls:     CAL_DAY
c----------------------------------------------------------------------
c --- Access module data
      use header
      use inpdata
      use userinf

      implicit none

c --- Declare arguments
      integer :: iolst,ioinp,k,ierr,ieof
      integer :: month, day
      real :: ozone

c --- Locals

      call cal_day(iolst,'TOGREGOR',nyre(1),month,day,njule(1)
     & ,ierr)

      ozone=ozmonth(month)

      end   
c-----------------------------------------------------------------------
      subroutine olmcalc2(hroz,iolst)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 230120            OLMCALC2
c               C. DesAutels, Exponent Inc.
c
c     Level 220403 to 230120
c
c Updates
c --- 7.0.0 to 7.0.1
c          220403 - 230120 : Updated calculated variable definitions to 
c                            double precision to avoid overflow
c
c --- PURPOSE:  Calculate NO2 concentrations based on Ozone Limited method
c               for one time period from 'k' sources.  Store output NO2
c               concentrations in output array.  Ozone sources do not compete
c
c --- INPUT:
c          HROZ  - integer - ozone concentration used for OLM calculations (ppm)
c          IOLST - integer - File unit number for output List file
c
c --- OUTPUT:
c
c --- OLMCALC2 called by: COMPOLM
c --- OLMCALC2 calls: none    
c----------------------------------------------------------------------
c --- Access module data
      use header
      use inpdata
      use userinf

      implicit none

c --- Declare arguments
      integer :: iolst,ioinp,k,ierr,ieof
      real :: hroz

c --- Locals
      integer :: is,nsources,isp,isin,i,j
      integer :: nxy,ncmprs,ione
      character(len=15) :: name
      double precision noxconc,noconc,ozlim

c --- Initializations
      ierr=0

c --- Assign input ratios to specific sources. HALT if ratio not provided
      do k=1,nfiles
       do is=1,nsrc(k)

c --- For MSOURCE=0 (no source contribution) get names from header
        if(msource(k).eq.0) then
         do i=1,nsrctype(k)
          if(nsbytype(i,k).eq.1) then
           name=csrcnam(1,i,k)(1:15)
          endif
         enddo    
c --- otherwise, get source name from hourly record.
        else
         name=sname(is,k)(1:15)
        endif

        ierr=1
        do isin=1,nsrc(nfiles+1)
         if(name.EQ.insource(isin)) then
          ratio(is,k)=no2nox(isin)
c          write(iolst,*)name,insource(isin),ratio(is,k),is,isin,k
          ierr=0
         endif
        enddo
        if(ierr.EQ.1) then
         write(*,*)'Halted in OLMCALC, see list file'
         write(iolst,*)'CALPUFF source ratio not found in input file'
         write(iolst,*)'Source ',name
         write(iolst,*)'Input ',insource
         stop
        endif
       enddo
      enddo

c --- Gridded receptors
      if(lsamp(1)) then
       do isp=1,nspout(nfiles+1)
        do i=1,ngx(nfiles+1)
         do j=1,ngy(nfiles+1)
          do is=1,nsrc(nfiles+1)
           datag(i,j,isp,1,nfiles+1)=0
          enddo
         enddo
        enddo
       enddo

       do i=1,ngx(1)
        do j=1,ngy(1)
         do isp=1,nspout(1)
          do k=1,nfiles 
           do is=1,nsrc(k)
            if (cspecg(isp,k).EQ.'NOX           1') then
c ---        Species record being read is identified as NOx
             noxconc=datag(i,j,isp,is,k)

c ---        transfer NO2 emitted directly from the stack based on the in stack ratio to the output array
             noconc=(1-ratio(is,k))*noxconc
c             write(iolst,*)'G-SUMMARY',is,k,ratio(is,k),noxconc,noconc
              
c ---        Calculate the moles of available ozone divided by moles of NO.
c ---        conversion is the minimum of the ozone ratio or Equilibrium Ratio  
c ---        Add converted NO2 to the NO2 derived from the In Stack Ratio           
             if(datag(i,j,isp,is,k).ne.0) then
              ozlim=(hroz)/(24.45*noconc*1000000/30.01)
             else
              ozlim=0
             endif
c             write(iolst,*)'LIMITS',hroz,noconc,ozlim,equil
             datag(i,j,isp,1,nfiles+1)=datag(i,j,isp,1,nfiles+1)
     &         +min(ozlim*noconc+ratio(is,k)*noxconc,equil*noxconc)
             cspecg(isp,nfiles+1)='NO2           1'
            else
             datag(i,j,isp,1,nfiles+1)=datag(i,j,isp,1,nfiles+1)
     &                      +datag(i,j,isp,is,k)
             cspecg(isp,nfiles+1)=cspecg(isp,k)
            endif
           enddo
          enddo
         enddo
        enddo
       enddo
      endif

c --- Discrete receptors
      if(ldisc(nfiles+1)) then
       do i=1,ndrec(nfiles+1)
        do isp=1,nspout(nfiles+1)
         do is=1,nsrc(nfiles+1)
          datad(i,isp,1,nfiles+1)=0
         enddo
        enddo
       enddo

       do i=1,ndrec(1)
        do isp=1,nspout(1)
         do k=1,nfiles
          do is=1,nsrc(k)
           if (cspecd(isp,k).EQ.'NOX           1') then

c ---        Species record being read is identified as NOx
             noxconc=datad(i,isp,is,k)

             noconc=(1-ratio(is,k))*noxconc
c             write(iolst,*)'SUMMARY',is,k,ratio(is,k),noxconc,noconc
              
c ---        Calculate the moles of available ozone divided by moles of NO.
c ---        conversion is the minimum of the ozone ratio or Equilibrium Ratio  
c ---        Add converted NO2 to the NO2 derived from the In Stack Ratio           
             if(datad(i,isp,is,k).ne.0) then
              ozlim=(hroz)/(24.45*noconc*1000000/30.01)
             else
              ozlim=0
             endif
c             write(iolst,*)'LIMITS',hroz,noconc,ozlim,equil
             datad(i,isp,1,nfiles+1)=datad(i,isp,1,nfiles+1)
     &        +min(ozlim*noconc+ratio(is,k)*noxconc,equil*noxconc)

            cspecd(isp,nfiles+1)='NO2           1'
           else
             datad(i,isp,1,nfiles+1)=datad(i,isp,1,nfiles+1)
     &                             +datad(i,isp,is,k)
             cspecd(isp,nfiles+1)=cspecd(isp,k)
           endif
          enddo
         enddo
        enddo
       enddo
      endif

c --- Complex terrain receptors
      if(lctsg(1)) then
       do i=1,nctrec(nfiles+1)
        do isp=1,nspout(nfiles+1)
         do is=1,nsrc(nfiles+1)
          datact(i,isp,1,nfiles+1)=0
         enddo
        enddo
       enddo

       do i=1,nctrec(1)
        do isp=1,nspout(1)      
         do k=1,nfiles
          do is=1,nsrc(k)
           if (cspecct(isp,k).EQ.'NOX           1') then

c ---        Species record being read is identified as NOx
             noxconc=datact(i,isp,is,k)

             noconc=(1-ratio(is,k))*noxconc
              
c ---        Calculate the moles of available ozone divided by moles of NO.
c ---        conversion is the minimum of the ozone ratio or Equilibrium Ratio  
c ---        Add converted NO2 to the NO2 derived from the In Stack Ratio           
             if(datact(i,isp,is,k).ne.0) then
              ozlim=(hroz)/(24.45*noconc*1000000/30.01)
             else
              ozlim=0
             endif
             datact(i,isp,1,nfiles+1)=datact(i,isp,1,nfiles+1)
     &         +min(ozlim*noconc+ratio(is,k)*noxconc,equil*noxconc)
             cspecct(isp,nfiles+1)='NO2           1'
           else
             datact(i,isp,1,nfiles+1)=datact(i,isp,1,nfiles+1)
     &                        +datact(i,isp,is,k)
             cspecct(isp,nfiles+1)=cspecct(isp,k)
           endif
          enddo
         enddo
        enddo
       enddo
      endif


      return
      end

c-----------------------------------------------------------------------
      subroutine olmcalc1(hroz,iolst)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 230120            OLMCALC1
c               C. DesAutels, Exponent Inc.
c
c --- PURPOSE:  Calculate NO2 concentrations based on Ozone Limited method
c               for one time period from 'k' sources.  Store output NO2
c               concentrations in output array. Source compete for ozone.
c Updates
c --- 7.0.0 to 7.0.1
c          220403 - 230120 : Updated calculated variable definitions to 
c                            double precision to avoid overflow
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          HROZ  - integer - Ozone concentration used for OLM calculations (ppm)
c
c --- OUTPUT:
c
c --- OLMCALC1 called by: COMPOLM 
c --- OLMCALC1 calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header
      use inpdata
      use userinf

      implicit none

c --- Declare arguments
      integer :: iolst,ioinp,k,ierr,ieof,inox
      real :: hroz

c --- Locals
      integer :: is,nsources,isp,isin,i,j
      integer :: nxy,ncmprs,ione
      character(len=15) :: name
      double precision noxconc,noconc,ozlim,noxconctot

c --- Initializations
      ierr=0

c --- Assign input ratios to specific sources. HALT if ratio not provided
      do k=1,nfiles
       do is=1,nsrc(k)
c --- For MSOURCE=0 (no source contribution) get names from header
        if(msource(k).eq.0) then
         do i=1,nsrctype(k)
          if(nsbytype(i,k).eq.1) then
           name=csrcnam(1,i,k)(1:15)
          endif
         enddo    
c --- otherwise, get source name from hourly record.
        else
         name=sname(is,k)(1:15)
        endif
        ierr=1
        do isin=1,nsrc(nfiles+1)
         if(name.EQ.insource(isin)) then
          ratio(is,k)=no2nox(isin)
c          write(iolst,*)name,insource(isin),ratio(is,k),is,isin,k
          ierr=0
         endif
        enddo
        if(ierr.EQ.1) then
         write(*,*)'Halted in OLMCALC, see list file'
         write(iolst,*)'CALPUFF source ratio not found in input file'
         write(iolst,*)'Source ',name
         write(iolst,*)'Input ',insource
         stop
        endif
       enddo
      enddo

c --- Gridded receptors
      if(lsamp(k)) then 
       do i=1,ngx(nfiles+1)
        do j=1,ngy(nfiles+1)
         do isp=1,nspout(nfiles+1)
          do is=1,nsrc(nfiles+1)
           datag(i,j,isp,1,nfiles+1)=0
          enddo
         enddo
        enddo
       enddo

       do i=1,ngx(1)
        do j=1,ngy(1)
         do isp=1,nspout(1)      
          do k=1,nfiles
           do is=1,nsrc(k)
            if (cspecg(isp,k).EQ.'NOX           1') then
             inox=isp
c ---        Species record being read is identified as NOx
             noxconc=datag(i,j,isp,is,k)

c ---        transfer NO2 emitted directly from the stack based on the in stack ratio to the output array
             datag(i,j,isp,1,nfiles+1)=datag(i,j,isp,1,nfiles+1)
     &                                   +ratio(is,k)*noxconc
             cspecg(isp,nfiles+1)='NO2           1'
             noconc=noconc+(1-ratio(is,k))*noxconc
             noxconctot=noxconctot+noxconc
c             write(iolst,*)'G-SUMMARY',is,k,ratio(is,k),noxconc,noconc              
            else
             datag(i,j,isp,1,nfiles+1)=datag(i,j,isp,1,nfiles+1)
     &                      +datag(i,j,isp,is,k)
             cspecg(isp,nfiles+1)=cspecg(isp,k)
            endif
           enddo
          enddo
         enddo

c ---    After all file and sources are read:
c ---    Calculate the moles of available ozone divided by moles of NO.
c ---    conversion is the minimum of the ozone ratio or Equilibrium Ratio  
c ---    Add converted NO2 to the NO2 derived from the In Stack Ratio           
         if(noconc.ne.0) then
          ozlim=(hroz)/(24.45*noconc*1000000/30.01)
         else
          ozlim=0
         endif
c         write(iolst,*)'LIMITS',hroz,noconc,ozlim,equil
         datag(i,j,inox,1,nfiles+1)=datag(i,j,inox,1,nfiles+1)
     &                      +ozlim*noconc
c ---    Limit to Equilibrium conversion rate is applicable
         datag(i,j,inox,1,nfiles+1)=min(datag(i,j,inox,1,nfiles+1),
     &          equil*noxconctot)
         noconc=0
         noxconctot=0
        enddo
       enddo
      endif

c --- Discrete receptors
      if(ldisc(nfiles+1)) then
       do i=1,ndrec(nfiles+1)
        do isp=1,nspout(nfiles+1)
         do is=1,nsrc(nfiles+1)
          datad(i,isp,1,nfiles+1)=0
         enddo
        enddo
       enddo

       do i=1,ndrec(1)
        do isp=1,nspout(1)      
         do k=1,nfiles
          do is=1,nsrc(k)
           if (cspecd(isp,k).EQ.'NOX           1') then
c ---        Species record being read is identified as NOx
             inox=isp
             noxconc=datad(i,isp,is,k)

c ---        transfer NO2 emitted directly from the stack based on the in stack ratio to the output array
             datad(i,isp,1,nfiles+1)=datad(i,isp,1,nfiles+1)
     &                                 +ratio(is,k)*noxconc
             cspecd(isp,nfiles+1)='NO2           1'
             noconc=noconc+(1-ratio(is,k))*noxconc
             noxconctot=noxconctot+noxconc
c             write(iolst,*)'SUMMARY',is,k,ratio(is,k),noxconc,noconc
              
           else
             datad(i,isp,1,nfiles+1)=datad(i,isp,1,nfiles+1)
     &                             +datad(i,isp,is,k)
             cspecd(isp,nfiles+1)=cspecd(isp,k)
           endif
          enddo
         enddo
        enddo

c ---   After all file and sources are read:
c ---   Calculate the moles of available ozone divided by moles of NO.
c ---   conversion is the minimum of the ozone ratio or Equilibrium Ratio  
c ---   Add converted NO2 to the NO2 derived from the In Stack Ratio           
        if(noconc.ne.0) then
         ozlim=(hroz)/(24.45*noconc*1000000/30.01)
        else
         ozlim=0
        endif
c        write(iolst,*)'LIMITS',hroz,noconc,ozlim,equil
        datad(i,inox,1,nfiles+1)=datad(i,inox,1,nfiles+1)
     &                      + ozlim*noconc
c ---    Limit to Equilibrium conversion rate is applicable
        datad(i,inox,1,nfiles+1)=min(datad(i,inox,1,nfiles+1),
     &          equil*noxconctot)
        noconc=0
        noxconctot=0
       enddo
      endif

c --- Complex terrain receptors
      if(lctsg(k)) then
       do i=1,nctrec(nfiles+1)
        do isp=1,nspout(nfiles+1)
         do is=1,nsrc(nfiles+1)
          datact(i,isp,1,nfiles+1)=0
         enddo
        enddo
       enddo

       do i=1,nctrec(1)
        do isp=1,nspout(1)      
         do k=1,nfiles
          do is=1,nsrc(k)
           if (cspecct(isp,k).EQ.'NOX           1') then
             inox=isp
c ---        Species record being read is identified as NOx
             noxconc=datact(i,isp,is,k)

c ---        transfer NO2 emitted directly from the stack based on the in stack ratio to the output array
             datact(i,isp,1,nfiles+1)=datact(i,isp,1,nfiles+1)
     &                   +ratio(is,k)*noxconc
             cspecct(isp,nfiles+1)='NO2           1'
c ---        sum up balance as NO for later calculation of NO2 based on OLM
             noconc=noconc+(1-ratio(is,k))*noxconc
             noxconctot=noxconctot+noxconc
           else
             datact(i,isp,1,nfiles+1)=datact(i,isp,1,nfiles+1)
     &                        +datact(i,isp,is,k)
             cspecct(isp,nfiles+1)=cspecct(isp,k)
           endif
          enddo
         enddo
        enddo

c ---   After all file and sources are read:
c ---   Calculate the moles of available ozone divided by moles of NO.
c ---   conversion is the minimum of the ozone ratio or Equilibrium Ratio  
c ---   Add converted NO2 to the NO2 derived from the In Stack Ratio           
        if(noconc.ne.0) then
         ozlim=(hroz)/(24.45*noconc*1000000/30.01)
        else
         ozlim=0
        endif
        datact(i,inox,1,nfiles+1)=datact(i,inox,1,nfiles+1)
     &                        +ozlim*noconc
c ---   Limit to Equilibrium conversion rate is applicable
        datact(i,inox,1,nfiles+1)=min(datact(i,inox,1,nfiles+1),
     &       equil*noxconctot)    
        noconc=0
        noxconctot=0
       enddo
      endif


      return
      end

c-----------------------------------------------------------------------
      subroutine armcalc(iolst)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403            ARMCALC
c               C. DesAutels, Exponent Inc.
c
c --- PURPOSE:  Calculate NO2 concentrations based on Ambient Ratio method
c               for one time period from 'k' sources.  Store output NO2
c               concentrations in output array.
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c
c --- OUTPUT:
c
c --- ARMCALC called by: COMPARM 
c --- ARMCALC calls:     non3
c----------------------------------------------------------------------
c --- Access module data
      use header
      use inpdata
      use userinf
      use armdata

      implicit none

c --- Declare arguments
      integer :: iolst,ioinp,k,ierr,ieof

c --- Locals
      integer :: is,nsources,isp,i,j
      integer :: nxy,ncmprs,ione,inox

      real :: noxconc,no2ratio,noxconc_ug

c --- Initializations
      ierr=0

c --- Gridded Receptors
      if(lsamp(1)) then
       do i=1,ngx(nfiles+1)
        do j=1,ngy(nfiles+1)
         do isp=1,nspout(nfiles+1)
          do is=1,nsrc(nfiles+1)
           datag(i,j,isp,1,nfiles+1)=0
          enddo
         enddo
        enddo
       enddo

      do i=1,ngx(1)
       do j=1,ngy(1)
        do isp=1,nspout(1)      
         do k=1,nfiles
          do is=1,nsrc(k)
           if (cspecg(isp,k).EQ.'NOX           1') then
c ---       Species record identified as NOx
            inox=isp
            noxconc=noxconc+datag(i,j,isp,is,k)
           else
            datag(i,j,isp,1,nfiles+1)=datag(i,j,isp,1,nfiles+1)
     &          +datag(i,j,isp,is,k)
            cspecg(isp,nfiles+1)=cspecg(isp,k)
           endif
          enddo
         enddo
         if (cspecg(isp,1).EQ.'NOX           1') then
          noxconc_ug=noxconc*1000000
          call ratiocalc(noxconc_ug,no2ratio)
          datag(i,j,inox,1,nfiles+1)=no2ratio*noxconc
          cspecg(inox,nfiles+1)='NO2           1'
          noxconc=0
         endif
        enddo
       enddo
      enddo
      endif

c --- Discrete Receptors
      if(ldisc(1)) then
       do i=1,ndrec(nfiles+1)
        do isp=1,nspout(nfiles+1)
         do is=1,nsrc(nfiles+1)
          datad(i,isp,1,nfiles+1)=0
         enddo
        enddo
       enddo

       do i=1,ndrec(1)
        do isp=1,nspout(1)
         do k=1,nfiles
          do is=1,nsrc(k)
            if (cspecd(isp,k).EQ.'NOX           1') then
c ---      Species record identified as NOX and source is TOTAL
             inox=isp
             noxconc=noxconc+datad(i,isp,is,k)
           else
             datad(i,isp,1,nfiles+1)=datad(i,isp,1,nfiles+1)
     &          +datad(i,isp,is,k)
             cspecd(isp,nfiles+1)=cspecd(isp,k)
           endif
          enddo
         enddo
         if (cspecd(isp,1).EQ.'NOX           1') then
          noxconc_ug=noxconc*1000000
          call ratiocalc(noxconc_ug,no2ratio)
          datad(i,inox,1,nfiles+1)=no2ratio*noxconc
          cspecd(inox,nfiles+1)='NO2           1'
          noxconc=0
         endif
        enddo
       enddo
      endif

c --- Complex Receptors
      
      if(lctsg(1))then
       do i=1,nctrec(nfiles+1)
        do isp=1,nspout(nfiles+1)
         do is=1,nsrc(nfiles+1)
          datact(i,isp,1,nfiles+1)=0
         enddo
        enddo
       enddo

       do i=1,nctrec(1)
        do isp=1,nspout(1)      
         do k=1,nfiles
          do is=1,nsrc(k)
           if (cspecd(isp,k).EQ.'NOX           1') then
c ---       Species record identified as NOX
            inox=isp
            noxconc=noxconc+datact(i,isp,is,k)
           else
            datact(i,isp,1,nfiles+1)=datact(i,isp,1,nfiles+1)
     &         +datact(i,isp,is,k)
            cspecct(isp,nfiles+1)=cspecct(isp,k)
           endif
          enddo
         enddo
         if (cspecd(isp,k).EQ.'NOX           1') then
          noxconc_ug=noxconc*1000000
          call ratiocalc(noxconc_ug,no2ratio)
          datact(i,inox,1,nfiles+1)=no2ratio*noxconc
          cspecct(inox,nfiles+1)='NO2           1'
         endif
        enddo
       enddo
      endif

      return
      end

c-----------------------------------------------------------------------
      subroutine ratiocalc(noxconc,no2ratio)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403            RATIOCALC
c               C. DesAutels, Exponent Inc.
c
c --- PURPOSE:  Calculates ratio of emitted NOx converted to NO2 based on 
c               ARM polynomial profiles
c         
c
c --- INPUT:
c          NOXCONC - integer - Total NOx concentration at a specific receptor
c                              read and summed from listed CALPUFF concentration
c                              files
c
c --- OUTPUT:
c          NO2RATIO - integer - Calculated ratio of NOx converted to NO2 based on
c                               ARM polynomial equations
c
c --- RATIOCALC called by: ARMCALC
c --- RATIOCALC calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use header
      use inpdata
      use userinf
      use armdata

      implicit none

c --- Declare arguments
      real :: noxconc,no2ratio

      if (aprof.eq.1) then
       no2ratio=C0_All+C1_All*noxconc+C2_All*noxconc**2
     &          +C3_All*noxconc**3+C4_All*noxconc**4
     &          +C5_All*noxconc**5+C6_All*noxconc**6
       if(no2ratio.gt.maxratio) no2ratio=maxratio
       if(noxconc.gt.maxthresh) no2ratio=minratio
       if(no2ratio.lt.minratio) no2ratio=minratio
       if(noxconc.lt.minthresh) no2ratio=maxratio
             
      elseif (aprof.eq.2) then
       no2ratio=C0_Urb+C1_Urb*noxconc+C2_Urb*noxconc**2
     &          +C3_Urb*noxconc**3+C4_Urb*noxconc**4
     &          +C5_Urb*noxconc**5+C6_Urb*noxconc**6
       if(no2ratio.gt.maxratio) no2ratio=maxratio
       if(noxconc.gt.maxthresh) no2ratio=minratio
       if(no2ratio.lt.minratio) no2ratio=minratio
       if(noxconc.lt.minthresh) no2ratio=maxratio

      elseif (aprof.eq.3) then
       no2ratio=C0_Rrl+C1_Rrl*noxconc+C2_Rrl*noxconc**2
     &          +C3_Rrl*noxconc**3+C4_Rrl*noxconc**4
     &          +C5_Rrl*noxconc**5+C6_Rrl*noxconc**6
       if(no2ratio.gt.maxratio) no2ratio=maxratio
       if(noxconc.gt.maxthresh) no2ratio=minratio
       if(no2ratio.lt.minratio) no2ratio=minratio
       if(noxconc.lt.minthresh) no2ratio=maxratio

      elseif (aprof.eq.4) then
       no2ratio=C0_Ind+C1_Ind*noxconc+C2_Ind*noxconc**2
     &         +C3_Ind*noxconc**3+C4_Ind*noxconc**4
     &         +C5_Ind*noxconc**5+C6_Ind*noxconc**6
       if(no2ratio.gt.maxratio) no2ratio=maxratio
       if(noxconc.gt.maxthresh) no2ratio=minratio
       if(no2ratio.lt.minratio) no2ratio=minratio
       if(noxconc.lt.minthresh) no2ratio=maxratio

      elseif (aprof.eq.5) then
       no2ratio=C0_Cst+C1_Cst*noxconc+C2_Cst*noxconc**2
     &          +C3_Cst*noxconc**3+C4_Cst*noxconc**4
     &          +C5_Cst*noxconc**5+C6_Cst*noxconc**6
       if(no2ratio.gt.maxratio) no2ratio=maxratio
       if(noxconc.gt.maxthresh) no2ratio=minratio
       if(no2ratio.lt.minratio) no2ratio=minratio
       if(noxconc.lt.minthresh) no2ratio=maxratio

      elseif (aprof.eq.6) then
       no2ratio=C0_All+C1_EPA*noxconc+C2_EPA*noxconc**2
     &          +C3_EPA*noxconc**3+C4_EPA*noxconc**4
     &          +C5_EPA*noxconc**5+C6_EPA*noxconc**6

      elseif (aprof.eq.7) then
       no2ratio=afact*noxconc**bfact

      else
       write(*,*)'Invalid value for APROF: ',aprof
      endif

      return
      end

c-----------------------------------------------------------------------
      subroutine alloc_2(k)
c-----------------------------------------------------------------------
c
c --- CALMAX    Version: 7.0.1          Level: 220403           ALLOC_2
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Allocate arrays of header information that depend on
c               the number of CALPUFF files processed and other
c               configuration data, and all data arrays
c
c --- UPDATES:
c
c --- v1.1 (120915) to TNG-2.0.0 (140912)
c        1. Add CALPUFF dataset TNG-3.0
c
c --- v1.0 (120715) to v1.1 (120915)
c        1. Add flag array for time-period status
c
c --- INPUT:
c              k - integer - Number of CALPUFF files
c
c --- OUTPUT:
c
c --- ALLOC_2 called by: (main)
c --- ALLOC_2 calls:     none
c----------------------------------------------------------------------
c --- Access module data
      use userinf
      use header
      use inpdata

      implicit none

c --- Arguments
      integer :: k

c --- Locals
      integer :: i,n
      integer :: mxspout,mxsrc,mxsrc1,mxsrc2
      integer :: mxdrec,mxctrec,mxnx,mxny,mxrgrp

c --- Get maximum number of species-levels across files
      mxspout=1
      do i=1,k
         mxspout=MAX(mxspout,nspout(i))
      enddo

c --- Get maximum number of discrete receptors across files
      mxdrec=1
      do i=1,k
         mxdrec=MAX(mxdrec,ndrec(i))
      enddo

c --- Get maximum number of gridded receptors across files
      mxnx=1
      mxny=1
      do i=1,k
         mxnx=MAX(mxnx,ngx(i))
         mxny=MAX(mxny,ngy(i))
      enddo

c --- Get maximum number of CTSG receptors across files
      mxctrec=1
      do i=1,k
         mxctrec=MAX(mxctrec,nctrec(i))
      enddo

c --- Get maximum number of each source type across files
      mxsrc=1
      mxsrc1=1
      mxsrc2=1
      do i=1,k
         mxsrc=MAX(mxsrc,nsrc(i))
         do n=1,mxsrctype
            mxsrc1=MAX(mxsrc1,nsbytype(n,i))
         enddo
         if(msource(i).EQ.1) mxsrc2=MAX(mxsrc2,nsrc(i)+1)
      enddo

c --- Get maximum number of receptor groups across files
      mxrgrp=1
      do i=1,k
         mxrgrp=MAX(mxrgrp,nrgrp(i))
      enddo

c --- Character arrays for header
      allocate (csout(mxspout,k),csunit(mxspout,k))
      allocate (csrcnam(mxsrc1,mxsrctype,k))
      allocate (argrpnam(mxrgrp,k))

c --- Integer arrays for header
      allocate (ihill(mxctrec,k))
      allocate (irgrp(mxdrec,k))
      
c --- Real arrays for header
      allocate (xrec(mxdrec,k),yrec(mxdrec,k))
      allocate (zrec(mxdrec,k),htrec(mxdrec,k))
      allocate (xctr(mxctrec,k),yctr(mxctrec,k),zctr(mxctrec,k))


c --- Character arrays for data
      allocate (sname(mxsrc,k))
      allocate (ratio(mxsrc,k))
      allocate (cspecg(mxspout,k),cspecd(mxspout,k),cspecct(mxspout,k))

c --- Integer arrays for data
      allocate (istype(mxsrc,k),isnum(mxsrc,k))
      allocate (nyrb(k),njulb(k))
      allocate (nhrb(k),nsecb(k))
      allocate (nyre(k),njule(k))
      allocate (nhre(k),nsece(k))
      allocate (datetime_b(k),datetime_e(k))
      allocate (istatus(k))

c --- Real arrays for data
      allocate (sxkm(mxsrc,k),sykm(mxsrc,k))
      allocate (datag(mxnx,mxny,mxspout,mxsrc2,k))
      allocate (datad(mxdrec,mxspout,mxsrc2,k))
      allocate (datact(mxctrec,mxspout,mxsrc2,k))

c --- Real work arrays
      allocate (workg(mxnx,mxny))
      allocate (workd(mxdrec))
      allocate (workct(mxctrec))

      return
      end
c-----------------------------------------------------------------------
      subroutine putper(iolst,ioout,ierr)
c-----------------------------------------------------------------------
c
c --- CALNO2    Version: 7.0.1          Level: 220403            PUTPER
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Write data records for 1 time period
c
c               Receptor arrays must be dimensioned EXACTLY for the
c               number(s) of receptors in the CALPUFF file!
c
c --- INPUT:
c          IOLST - integer - File unit number for output List file
c          IOOUT - integer - File unit number for output CALPUFF file
c
c --- OUTPUT:
c           IERR - integer - Error flag (0 = No Error)
c
c --- PUTPER called by: COMPOLM, COMPARM
c --- PUTPER calls:     CMPRSIO, WRBDAT, DATE_HRSEC, TIME_INCR
c----------------------------------------------------------------------
c --- Access module data
      use header
      use armdata
      use inpdata
      use userinf

      implicit none

c --- Declare arguments
      integer :: iolst,ioout,ierr

c --- Locals
      integer :: ione,k,is,isp
      integer :: nxy,ncmprs

c --- Initializations
      ione=1
c --- Use header info for file 1, source 1
      k=1+nfiles
      is=1

c --- Number of gridded receptors
      nxy=ngx(k)*ngy(k)

      nyrb(k)=nyrb(1)
      njulb(k)=njulb(1)
      nhrb(k)=nhrb(1)
      nsecb(k)=nsecb(1)
      nyre(k)=nyre(1)
      njule(k)=njule(1)
      nhre(k)=nhre(1)
      nsece(k)=nsece(1)
      

c --- Record 1:  Time period
      write(ioout) nyrb(k),njulb(k),nhrb(k),nsecb(k),
     &     nyre(k),njule(k),nhre(k),nsece(k)

cQC
c      write(iolst,*) nyrb(k),njulb(k),nhrb(k),nsecb(k),
c     &     nyre(k),njule(k),nhre(k),nsece(k)

c --- Record 2:  Source information (Set to write TOTAL - no source contribution)
      write(ioout)0,1,'TOTAL           ',0.0,0.0
cQC
c      write(iolst,*)0,1,'TOTAL           ',0.0,0.0

c --- Output species-level loop
c -----------------------------
      do isp=1,nspout(k)
         if(lsamp(k)) then
c            cspecg(isp,k)=cspecg(isp,1)
            if(lcompr(k)) then
               ncmprs=nxy
               call CMPRSIO(iolst,ioout,'PUTR',
     &                      datag(1,1,isp,1,k),nxy,
     &                      workg,ncmprs,cspecg(isp,k),ierr)
               if(ierr.GT.0) return
            else
               call WRBDAT(ioout,cspecg(isp,k),
     &                     datag(1,1,isp,1,k),ngx(k),ngy(k))
            endif
         endif

         if(ldisc(k)) then
            if(lcompr(k))then
c               cspecd(isp,k)=cspecd(isp,1)
               ncmprs=ndrec(k)

               call CMPRSIO(iolst,ioout,'PUTR',
     &                      datad(1,isp,1,k),ndrec(k),
     &                      workd,ncmprs,cspecd(isp,k),ierr)
               if(ierr.GT.0) return

            else
               call WRBDAT(ioout,cspecd(isp,k),
     &                     datad(1,isp,1,k),ndrec(k),ione)
            endif
         endif

         if(lctsg(k)) then
            if(lcompr(k))then
               ncmprs=nctrec(k)
               call CMPRSIO(iolst,ioout,'PUTR',
     &                      datact(1,isp,1,k),nctrec(k),
     &                      workd,ncmprs,cspecct(isp,k),ierr)
               if(ierr.GT.0) return
            else
               call WRBDAT(ioout,cspecct(isp,k),
     &                     datact(1,isp,1,k),nctrec(k),ione)
            endif
         endif
      enddo

      return
      end

  