c-----------------------------------------------------------------------
      module armdata
c-----------------------------------------------------------------------
c --- CALNO2    Version: 7.0.1          Level: 140912          /ARMDATA/
c               Exponent Inc.
c --- |================================================================================|
c --- | Variables used in ARM calculations. Six NO2/NOx Ratio Profiles have been       |
c --- | defined based on 6th order polynomials. The constants for each of these        |
c --- | profiles are defined below.                                                    |
c --- |                                                                                |
c --- | The form of the equation being defined is:                                     |
c     |                                                                                |
c --- |  NO2/NOx = C0 + C1*NOx + C2*NOx^2 + C3*NOx^3 + C4*NOx^4 + C5*NOx^5 + C6*NOx^6  |
c --- |                                                                                |
c --- | Maximum and minimum ratios are also defined below which apply to all curves    |
c --- | NOx threshold values are listed above and below which the maximum and minimum  |
c --- | Ratios will alway apply                                                        |
c --- |                                                                                |
c-----|================================================================================|

      real C0_All,C1_All,C2_All,C3_All,C4_All,C5_All,C6_All
      real C0_Urb,C1_Urb,C2_Urb,C3_Urb,C4_Urb,C5_Urb,C6_Urb
      real C0_Rrl,C1_Rrl,C2_Rrl,C3_Rrl,C4_Rrl,C5_Rrl,C6_Rrl
      real C0_Ind,C1_Ind,C2_Ind,C3_Ind,C4_Ind,C5_Ind,C6_Ind
      real C0_Cst,C1_Cst,C2_Cst,C3_Cst,C4_Cst,C5_Cst,C6_Cst
      real C0_EPA,C1_EPA,C2_EPA,C3_EPA,C4_EPA,C5_EPA,C6_EPA
      real maxratio, minratio, maxthresh,minthresh

c --- Data Values Used for Polynomial Profiles:
c ---
c --- PROFILE: ALL
      data C0_All/+1.4217e+00/
      data C1_All/-9.0043e-03/
      data C2_All/+2.8689e-05/
      data C3_All/-5.1310e-08/
      data C4_All/+6.2556e-11/
      data C5_All/-5.5299e-14/
      data C6_All/+2.4169e-17/
c ---
c --- PROFILE: URBAN
      data C0_Urb/+1.4081e+00/
      data C1_Urb/-8.4309e-03/
      data C2_Urb/+2.2008e-05/
      data C3_Urb/-1.8692e-08/
      data C4_Urb/-1.4082e-11/
      data C5_Urb/+2.976e-14/
      data C6_Urb/-1.1526e-17/
c ---
c --- PROFILE: RURAL
      data C0_Rrl/+7.0908e-01/
      data C1_Rrl/+1.8014e-02/
      data C2_Rrl/-4.0219e-04/
      data C3_Rrl/+3.1248e-06/
      data C4_Rrl/-1.1639e-08/
      data C5_Rrl/+2.0910e-11/
      data C6_Rrl/-1.4534e-14/
c ---
c --- PROFILE: INDUSTRIAL
      data C0_Ind/+9.7054e-01/
      data C1_Ind/+2.7563e-03/
      data C2_Ind/-8.0316e-05/
      data C3_Ind/+4.4204e-07/
      data C4_Ind/-1.0885e-09/
      data C5_Ind/+1.2663e-12/
      data C6_Ind/-5.6578e-16/
c ---
c --- PROFILE: Coastal
      data C0_Cst/+1.4097e+00/
      data C1_Cst/-8.6617e-03/
      data C2_Cst/+2.6443e-05/
      data C3_Cst/-5.9049e-08/
      data C4_Cst/+1.3159e-10/
      data C5_Cst/-2.0684e-13/
      data C6_Cst/+1.3132e-16/
c ---
c --- PROFILE: EPA
      data C0_EPA/+1.2441e+00/
      data C1_EPA/-2.7383e-03/
      data C2_EPA/-5.6062e-06/
      data C3_EPA/+3.4555e-08/
      data C4_EPA/-5.8345e-11/
      data C5_EPA/+4.2795e-14/
      data C6_EPA/-1.1723e-17/

c ---
c --- Maxmimum and minimum ratios - 
c ---        All computed ratios higher than the max will be set to the max
c ---        All computed ratios lower than the min will be set to the min

      data maxratio/0.9/
      data minratio/0.2/

c ---
c --- Maximum and minimum thresholds - 
c ---        All NOx concentrations greater than the maximum threshold will use the maxratio
c ---        All NOx concentrations less than the minimum threshold will use the minratio

      data maxthresh/600.0/
      data minthresh/40.0/

      end module armdata

c-----------------------------------------------------------------------
      module header
c-----------------------------------------------------------------------
c --- CALMAX    Version: 7.0.1          Level: 140912          /HEADER/
c               D. Strimaitis, Exponent Inc.
c
c --- Header Data from input CALPUFF CONC/WET/DRY files
c --- Use variable names as listed in the CALPUFF Users Guide
c
c --- Use arrays for ALL variables so that information from multiple
c --- data files can be stored (allocate)
c
c --- UPDATES:
c
c --- v1.0 (120715) to TNG-2.0.0 (140912)
c        1. Add CALPUFF dataset TNG-3.0
c-----------------------------------------------------------------------

      logical, allocatable :: lcompr(:),lsamp(:),ldisc(:),lctsg(:),
     &                        lspunit(:)                                 ! (files)

      character*12,  allocatable :: cmodel(:),ver(:),level(:)            ! (files)
      character*4,   allocatable :: utmhem(:)                            ! (files)
      character*8,   allocatable :: pmap(:),datum(:),axbtz(:)            ! (files)
      character*12,  allocatable :: daten(:)                             ! (files)
      character*15,  allocatable :: csout(:,:)                           ! (spec-lev,files)
      character*16,  allocatable :: clat0(:),clon0(:),clat1(:),clat2(:)  ! (files)
      character*16,  allocatable :: dataset(:),dataver(:)                ! (files)
      character*16,  allocatable :: csunit(:,:)                          ! (spec-lev,files)

      character*16,  allocatable :: csrcnam(:,:,:)                       ! (sources,types,files)

      character*64,  allocatable :: datamod(:)                           ! (files)
      character*80,  allocatable :: title(:,:)                           ! (3,files)
      character*80,  allocatable :: argrpnam(:,:)                        ! (drec groups,files)

      integer, allocatable :: ncom(:),ibyr(:),ibjul(:),ibhr(:),ibsec(:), ! (files)
     &                        irlg(:),iavg(:),nsecdt(:),
     &                        nxm(:),nym(:),nz(:),nssta(:),
     &                        ibcomp(:),iecomp(:),jbcomp(:),jecomp(:),
     &                        ibsamp(:),jbsamp(:),iesamp(:),jesamp(:),
     &                        meshdn(:),nsrctype(:),msource(:),
     &                        ndrec(:),nrgrp(:),nctrec(:),nspout(:),
     &                        i2dmet(:),iutmzn(:),ihill(:,:),
     &                        nsrc(:),ngx(:),ngy(:)
      integer, allocatable :: nsbytype(:,:)                              ! (types,files)
      integer, allocatable :: irgrp(:,:)                                 ! (drecs,files)
      integer :: mxsrctype
      
      real, allocatable :: xbtz(:),dxkm(:),dykm(:),xorigkm(:),           ! (files)
     &                     yorigkm(:),feast(:),fnorth(:),rnlat0(:),
     &                     relon0(:),xlat1(:),xlat2(:),
     &                     xrec(:,:),yrec(:,:),zrec(:,:),htrec(:,:),     ! (drecs,files)
     &                     xctr(:,:),yctr(:,:),zctr(:,:)                 ! (ctrecs,files)

c --- Assignments
c ---------------

c --- Current number of source-types
      data mxsrctype/13/

      end module header


c-----------------------------------------------------------------------
      module userinf
c-----------------------------------------------------------------------
c --- CALMAX    Version: 7.0.1          Level: 120915         /USERINF/
c               D. Strimaitis, Exponent Inc.
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Change pathname variables from C*132 to C*256
c        2. Add switch for processing individual sources in file or
c           just the total
c        3. Change the calendar-day output file to a time-block period
c           file
c
c --- Control file information
c-----------------------------------------------------------------------

      logical :: lbinout,lcfiles,ltotal,locomp

      character(len=256), allocatable :: inpfile(:)
      character(len=15), allocatable :: insource(:)
      character(len=256)              :: lstfile, ozfile, binfile

      integer, allocatable :: ioinparray(:)
      integer :: nfiles
      integer :: mode, nsource, ocomp
      integer :: ozsrc, aprof

      real :: equil
      real :: ozmonth(12)
      real :: afact, bfact
      real, allocatable :: no2nox(:)
      real, allocatable :: ratio(:,:)

c --- Calculated
      integer (kind=8) :: start_datetime,end_datetime
      integer :: nperiods

c --- Establish initial (default) values
c --- Always make BIN file
      data lbinout/.TRUE./
      data lcfiles/.FALSE./
      data ltotal/.FALSE./

      data ocomp/1/
      data equil/0.9/

      end module userinf


c-----------------------------------------------------------------------
      module inpdata
c-----------------------------------------------------------------------
c --- CALMAX    Version: 7.0.1          Level: 120915            /DATA/
c               D. Strimaitis, Exponent Inc.
c
c --- Data from input CALPUFF CONC/WET/DRY files
c --- Use variable names as listed in the CALPUFF Users Guide
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Add flag array for time-period status
c            istatus=-1   start-time of data younger than current period
c            istatus= 0   start-time of data within current period
c            istatus= 1   start-time of data older than current period
c            istatus= 9   no data reached end of file
c-----------------------------------------------------------------------

      character(len=16), allocatable :: sname(:,:)                      ! (sources,files)
      character(len=15), allocatable :: cspecg(:,:)                     ! (spec-lev,files)
      character(len=15), allocatable :: cspecd(:,:)                     ! (spec-lev,files)
      character(len=15), allocatable :: cspecct(:,:)                    ! (spec-lev,files)

      integer, allocatable :: istype(:,:),isnum(:,:)                    ! (sources,files)
      integer, allocatable :: nyrb(:),njulb(:),nhrb(:),nsecb(:)         ! (files)
      integer, allocatable :: nyre(:),njule(:),nhre(:),nsece(:)         ! (files)

      integer(kind=8), allocatable  :: datetime_b(:),datetime_e(:)      ! (files)
      integer, allocatable :: istatus(:)                                ! (files)

      real, allocatable :: sxkm(:,:),sykm(:,:)                          ! (sources,files)
      real, allocatable :: datag(:,:,:,:,:)                             ! (ngx,ngy,spec-lev,sources,files)
      real, allocatable :: datad(:,:,:,:)                               ! (ndrec,spec-lev,sources,files)
      real, allocatable :: datact(:,:,:,:)                              ! (nctrec,spec-lev,sources,files)

      real, allocatable :: workg(:,:)                                   ! (ngx,ngy)
      real, allocatable :: workd(:)                                     ! (ndrec)
      real, allocatable :: workct(:)                                    ! (nctrec)

      end module inpdata

