c----------------------------------------------------------------------
c --- UTILS -- UTILITY SUBROUTINES
c----------------------------------------------------------------------
c
c --- UTILS      Version: 1.1      Level: 120915
c
c --- UPDATES:
c --- v1.0 (120518) to v1.1 (120915)
c        1. Change file pathname variables from C*132 to C*256
c           MODIFIED: GETCMD
c
c----------------------------------------------------------------------
c --- System Interface
c      subroutine getcmd
c --- Data I/O
c      subroutine cmprsio
c      subroutine wrbdat
c      subroutine rdbdat
c --- String Manipulations (Control File Inputs)
c      subroutine mk_case
c      subroutine rm_blank
c      subroutine delim
c --- Date/Time Manipulations
c      subroutine utc_zone
c      subroutine date_hrsec
c      subroutine date_hr
c      subroutine cal_day
c      subroutine time_incr
c      subroutine time_diff

c----------------------------------------------------------------------
      subroutine getcmd(arg1)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 1.1            Level: 120518           GETCMD
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Get first command line argument
c
c               This is system and compiler-dependent, configured
c               here for the Lahey compiler for the Windows operating
c               system.  Modify this subroutine to configure for
c               other compilations.  For example, the Lahey GETCL
c               can be replaced with the GETARG call used by other
c               systems, with suitable declarations.  Consult your
c               FORTRAN users guide for details.
c
c --- UPDATES:
c --- v1.0 (120518) to v1.1 (120915)
c        1. Change default pathname variable from C*132 to C*256
c
c --- INPUTS:
c           ARG1 - character* - Default command line argument #1
c
c --- OUTPUTS:
c           ARG1 - character* - Returned command line argument #1
c                               (updated if argument provided)
c
c --- GETCMD called by:  ()
c --- GETCMD calls:      GETCL - Lahey system routine
c----------------------------------------------------------------------
      implicit none

      character(len=*)   :: arg1
      character(len=256) :: cdeflt

c --- Assign default
      cdeflt=arg1

c --- Read current argument (if any) from the command line
      call GETCL(arg1)

c --- Pass default when no command line arguments are given
      if(arg1(1:1) .EQ. ' ') arg1=cdeflt

      return
      end

c----------------------------------------------------------------------
      subroutine cmprsio(iolst,iodat,action,xdata,ndata,xcmprs,ncmprs,
     &                   clabel,ierr)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 1.1            Level: 120518          CMPRSIO
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Read & expand a compressed data record from binary file
c               or compress & write data record to binary file
c               (Compression replaces consecutive zeros with a single
c               negative count of the number of zeros)
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c               IODAT - integer   - Unit number of binary data file
c              ACTION - char*4    - Action to perform (GETR or PUTR)
c
c --- OUTPUT:
c                IERR - integer   - Error flag (0 = No Error)
c
c --- INPUT/OUTPUT: (depends on ACTION)
c        XDATA(ndata) - real arr. - Array of uncompressed data
c               NDATA - integer   - Number of values in full array
c      XCMPRS(ncmprs) - real arr. - Array compressed data
c              NCMPRS - integer   - Number of words in compressed
c                                   data record
c              CLABEL - char*     - Character record ID
c
c --- CMPRSIO called by: ()
c --- CMPRSIO calls:     WRBDAT
c----------------------------------------------------------------------
      implicit none

c --- Calling routine is responsible for string length
      character(len=*) :: clabel
      character*4 :: action

      real :: xcmprs(ncmprs),xdata(ndata)
      integer :: iolst,iodat,ndata,ncmprs,ierr
      integer :: i,j,nxpnd,nzero

      ierr=0

      if(action.EQ.'GETR') then
c ---    Initialization
         nxpnd=0

c ---    Read a compressed data record
         read(iodat) clabel,xcmprs

c ---    Expand the data to physical representation
         do i=1,ncmprs
            if(xcmprs(i).GT.0.0)then
c ---          Store value
               nxpnd=nxpnd+1
               xdata(nxpnd)=xcmprs(i)
            else
c ---          Store number of zeros indicated
               do j=1,NINT(-xcmprs(i))
                  nxpnd=nxpnd+1
                  xdata(nxpnd)=0.0
               enddo
            endif
         enddo

c ---    QA checks
         if(nxpnd.NE.ndata) then
            write(iolst,*)'ERROR in CMPRSIO -- Invalid expanded array '
            write(iolst,*)'   Number of words expected: ',ndata
            write(iolst,*)'   Number of words found   : ',nxpnd
            ierr=1
            return
         endif

      elseif(action.EQ.'PUTR') then
c ---    Initialization
         nzero=0
         ncmprs=0

c ---    Loop over all words in data array
         do i=1,ndata

c ---       QA:  Negative data values cannot be used here
            if(xdata(i).LT.0.0) then
               write(iolst,*)'ERROR in CMPRSIO -- Negative data '//
     &              'values cannot be used with COMPRESSION option'
               write(iolst,*)'Data ID, index, value = ',clabel,
     &                        i,xdata(i)
               ierr=1
               return
            endif

            if(xdata(i).EQ.0.0) then
c ---          Just increment zero counter for now
               nzero=nzero+1
            elseif(nzero.EQ.0) then
c ---          Non-zero data and no preceding zeros: Place into output
               ncmprs=ncmprs+1
               xcmprs(ncmprs)=xdata(i)
            else
c ---          Non-zero data with preceding zeros
c ---          Write and reset compression number
               ncmprs=ncmprs+1
               xcmprs(ncmprs)=-(FLOAT(nzero)+0.0001)
               nzero=0
c ---          Write current non-zero value
               ncmprs=ncmprs+1
               xcmprs(ncmprs)=xdata(i)
            endif
         enddo

c ---    Treat case of last array value = zero
         if(nzero.GT.0) then
            ncmprs=ncmprs+1
            xcmprs(ncmprs)=-(FLOAT(nzero)+0.0001)
         endif

c ---    Write header and compressed data records
         write(iodat) ncmprs
         call WRBDAT(iodat,clabel,xcmprs,ncmprs,1)

      else
         write(iolst,*)'ERROR in CMPRSIO -- Invalid ACTION '
         write(iolst,*)'   ACTION expected: GETR or PUTR'
         write(iolst,*)'   ACTION found   : ',action
         ierr=1
         return
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine wrbdat(io,clabel,data,n,m)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 1.1            Level: 120518           WRBDAT
c               J. Scire, Exponent Inc.
c
c --- PURPOSE:  Write a gridded data record to binary file
c               (character identifier and a 2-D data array)
c
c --- INPUT:
c              IO - integer     - Unit no. of output file
c          clabel - character*  - Data label
c       DATA(n,m) - real arr.   - Data array
c             N,M - integer     - Array size
c
c --- OUTPUT:  none
c
c --- WRBDAT called by:  ()
c --- WRBDAT calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Calling routine is responsible for ID string length
      character(len=*) :: clabel
      real :: data(n,m)
      integer :: io,n,m

      write(io) clabel,data

      return
      end

c----------------------------------------------------------------------
      subroutine rdbdat(io,clabel,data,n,m)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 1.1            Level: 120518           RDBDAT
c               J. Scire, Exponent Inc.
c
c --- PURPOSE:  Read a gridded data record from binary file
c               (character identifier and a 2-D data array)
c
c --- INPUT:
c              IO - integer     - Unit no. of input file
c          clabel - character*  - Data label
c       DATA(n,m) - real arr.   - Data array
c             N,M - integer     - Array size
c
c --- OUTPUT:  none
c
c --- RDBDAT called by:  ()
c --- RDBDAT calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Calling routine is responsible for ID string length
      character(len=*) :: clabel
      real :: data(n,m)
      integer :: io,n,m

      read(io) clabel,data
      write(11,*) 'READ DATA'
      write(11,*) clabel,data

      return
      end

c----------------------------------------------------------------------
      subroutine utc_zone(iolst,action,char_tz,real_tz,intg_tz,ierr)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 1.1            Level: 120518         UTC_ZONE
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Converts UTC time zone designator between a character
c               string representation and either a real representation
c               or an integer representation
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c              ACTION - char*8    - Action to perform
c                                   (CHR2REAL or REAL2CHR)
c                                   (CHR2INTG or INTG2CHR)
c
c --- OUTPUT:
c                IERR - integer   - Error flag (0 = No Error)
c
c --- INPUT/OUTPUT:
c             CHAR_TZ - char*8    - Time zone as character string that
c                                   indicates time to add to UTC to
c                                   obtain local time: UTC+HHMM
c             REAL_TZ - real      - Time zone as real that indicates
c                                   hours to ADD to local time to
c                                   obtain UTC time
c             INTG_TZ - real      - Time zone as integer that indicates
c                                   hours to ADD to local time to
c                                   obtain UTC time
c
c --- UTC_ZONE called by:  ()
c --- UTC_ZONE calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      real :: real_tz
      integer :: intg_tz,iolst,ierr
      character(len=8) :: char_tz
      character(len=8) :: action

c --- Declare locals
      integer :: ihour,iminute

c --- Initializations
      ierr=0

      if(action.EQ.'CHR2REAL') then
c ---    (a) Convert from character to real representation
         if(INDEX(char_tz,'0000').GT.0) then
            real_tz=0.0
            return
         endif
c ---    2-digit hour with sign, 2-digit minutes
         read(char_tz(4:6),'(i3)') ihour
         read(char_tz(7:8),'(i2)') iminute
c ---    Apply sign to minutes
         iminute=SIGN(iminute,ihour)
c ---    Form real as time to ADD to local time to obtain UTC
         real_tz=-(FLOAT(ihour)+FLOAT(iminute)/60.)

      elseif(action.EQ.'CHR2INTG') then
c ---    (b) Convert from character to integer representation
         if(INDEX(char_tz,'0000').GT.0) then
            intg_tz=0
            return
         endif
c ---    2-digit hour with sign, 2-digit minutes
         read(char_tz(4:6),'(i3)') ihour
         read(char_tz(7:8),'(i2)') iminute
c ---    QA:  minutes must be zero
         if(iminute.NE.0) then
            write(iolst,*)'ERROR in UTC_ZONE -- Invalid ACTION for zone'
            write(iolst,*)'   CHR2INTG requires zone with zero minutes'
            write(iolst,*)'   Zone found : ',char_tz
            ierr=1
            return
         endif
c ---    Form integer as hours to ADD to local time to obtain UTC
         intg_tz=-ihour

      elseif(action.EQ.'REAL2CHR') then
c ---    (c) Convert from real to character representation
c ---    Create string template as time ADDED to UTC to obtain local
         if(real_tz.EQ.0.0) then
            char_tz='UTC+0000'
            return
         elseif(real_tz.LT.0.0) then
            char_tz='UTC+0000'
         else
            char_tz='UTC-0000'
         endif
c ---    Extract minutes as integer
         iminute=NINT(60.*MOD(ABS(real_tz),1.0))
c ---    Extract unsigned hours as integer
         ihour=INT(ABS(real_tz))
c ---    Place in string
         write(char_tz(5:6),'(i2.2)') ihour
         write(char_tz(7:8),'(i2.2)') iminute

      elseif(action.EQ.'INTG2CHR') then
c ---    (d) Convert from integer to character representation
c ---    Create string template as time ADDED to UTC to obtain local
         if(intg_tz.EQ.0) then
            char_tz='UTC+0000'
            return
         elseif(intg_tz.LT.0) then
            char_tz='UTC+0000'
         else
            char_tz='UTC-0000'
         endif
c ---    Place unsigned hours in string
         write(char_tz(5:6),'(i2.2)') IABS(intg_tz)

      else
         write(iolst,*)'ERROR in UTC_ZONE -- Invalid ACTION '
         write(iolst,*)'   ACTION expected: CHR2REAL or REAL2CHR'
         write(iolst,*)'   ACTION found   : ',action
         ierr=1
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine mk_case(iolst,action,cvar,ierr)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 1.1            Level: 120518          MK_CASE
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Imposes uniform case on a string
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c              ACTION - char*     - Action to perform
c                                   (UPPER or LOWER)
c
c --- OUTPUT:
c                IERR - integer   - Error flag (0 = No Error)
c
c --- INPUT/OUTPUT:
c                CVAR - char*     - String
c
c --- MK_CASE called by:  ()
c --- MK_CASE calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: cvar,action
      integer :: iolst,ierr

c --- Declare locals
      integer :: iul,lo(2),hi(2),shift(2)
      integer :: nc,ia,ic

c --- Define ASCII table pointers for conversion to UPPER and LOWER
c ---         (UPPER,LOWER)
      data lo   / 97,65/
      data hi   /122,90/
      data shift/-32,32/

c --- Initializations
      ierr=0

c --- Set conversion
      if(TRIM(action).EQ.'UPPER') then
         iul=1
      elseif(TRIM(action).EQ.'LOWER') then
         iul=2
      else
         write(iolst,*)'ERROR in MK_CASE -- Invalid ACTION'
         write(iolst,*)'   ACTION expected: UPPER or LOWER'
         write(iolst,*)'   ACTION found   : ',action
         ierr=1
         return
      endif

c --- String length
      nc=LEN(cvar)

c --- Process string
      do ic=1,nc
         ia=IACHAR(cvar(ic:ic))
         if(ia.GE.lo(iul) .AND. ia.LE.hi(iul)) then
            ia=ia+shift(iul)
            cvar(ic:ic)=ACHAR(ia)
         endif
      enddo

      return
      end

c----------------------------------------------------------------------
      subroutine rm_blank(cvar)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 1.1            Level: 120518         RM_BLANK
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Moves all blanks to end of string
c
c --- INPUT:
c
c --- OUTPUT:
c
c --- INPUT/OUTPUT:
c                CVAR - char*     - String
c
c --- RM_BLANK called by:  ()
c --- RM_BLANK calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: cvar

c --- Declare locals
      integer :: nc,ic,ix

c --- Intrinsic fcn clears leading blanks
      cvar=ADJUSTL(cvar)

c --- March along string, overwriting any blanks
      nc=LEN_TRIM(cvar)
      ix=1
      do ic=1,nc
         if(cvar(ic:ic).NE.' ') then
            cvar(ix:ix)=cvar(ic:ic)
            ix=ix+1
         endif
      enddo
c --- Clear to end of string if needed
      if(ix.LE.nc) then
         do ic=ix,nc
            cvar(ic:ic)=' '
         enddo
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine delim(dlim,cvar,id1,id2,ieq,lprocess)
c----------------------------------------------------------------------
c
c --- UTILITY   Version: 1.1            Level: 120518            DELIM
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Determines if CVAR is a line containing a control-file
c               assignment between delimiters and if so, it returns
c               the position of the 2 delimiters and the equals sign
c
c --- INPUT:
c                DLIM - char*     - Delimiter (should be 1 character)
c                CVAR - char*     - String
c
c --- OUTPUT:
c                 ID1 - integer   - Position of 1st delimiter in CVAR
c                 ID2 - integer   - Position of last delimiter in CVAR
c                 IEQ - integer   - Position of '=' in CVAR
c            LPROCESS - logical   - True if a good assignment
c
c --- DELIM called by:  ()
c --- DELIM calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      logical          :: lprocess
      character(len=*) :: dlim,cvar
      integer          :: id1,id2,ieq

c --- Locals
      integer          :: idx,ieqx

      lprocess=.TRUE.

c --- Find position of markers
      ieq=0
      id1=INDEX(cvar,dlim)
      id2=INDEX(cvar,dlim,back=.TRUE.)
      if(id1.GT.0 .AND. id2.GT.id1) then
         ieq=INDEX(cvar(id1:id2),'=')
         if(ieq.GT.0) ieq=ieq+id1-1
      endif

      if(id2.GT.id1) then
c ---    Are there more assignments between these?
         ieqx=INDEX(cvar(ieq+1:id2),'=')
         idx =INDEX(cvar(id1+1:id2-1),dlim)
         if(idx.GT.0 .OR. ieqx.GT.0) then
            write(*,*)'ERROR in Control File'
            write(*,*)'Line contains too many assignments'
            write(*,*)'Only 1 assignment is expected'
            write(*,*)TRIM(cvar)
            write(*,*)'id1,id2,idx = ',id1,id2,idx
            write(*,*)'ieq,ieqx    = ',ieq,ieqx
            stop 'HALTED in DELIM'
         endif
      endif

c --- Must have an equals sign
      if(ieq.EQ.0) then
         lprocess=.FALSE.
         return
      endif

c --- Check first delimiter
      if(id1.EQ.0) then
         lprocess=.FALSE.
         return
      elseif(id2.EQ.0) then
c ---    One delimiter found
         write(*,*)'ERROR in Control File'
         write(*,*)'Line contains only 1 delimiter'
         write(*,*)'Assignment between delimiters is expected'
         write(*,*)TRIM(cvar)
         stop 'HALTED in DELIM'
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine date_hrsec(iolst,action,year,juld,hour,second,
     &                      datetime,ierr)
c----------------------------------------------------------------------
c --- UTILITY   Version: 1.1            Level: 120518       DATE_HRSEC
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Converts between individual date and time variables
c               and single integer*8 YYYYJJJHHSSSS variable
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c              ACTION - char*     - Action to perform
c                                   TOINDV returns components
c                                   TOSNGL returns single integer
c
c --- OUTPUT:
c                IERR - integer   - Error flag (0 = No Error)
c
c --- INPUT/OUTPUT:
c                YEAR - integer   - Year (YYYY)
c                JULD - integer   - Julian day (JJJ)
c                HOUR - integer   - Hour (HH)
c              SECOND - integer   - Second (SSSS)
c            DATETIME - integer*8 - Combined date & time (YYYYJJJHHSSSS)
c
c --- DATE_HRSEC called by:  ()
c --- DATE_HRSEC calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: action
      integer          :: iolst,year,juld,hour,second,ierr
      integer(kind=8)  :: datetime

c --- Locals
      logical lbreakup

c --- Initializations
      ierr=0

c --- Set conversion
      if(action.EQ.'TOSNGL') then
         lbreakup=.FALSE.
c ---    QA inputs
         if(year.GT.9999 .OR. year.LT.1) ierr=1
         if(juld.GT.366 .OR. juld.LT.1) ierr=1
         if(hour.GT.24 .OR. hour.LT.0) ierr=1
         if(second.GT.3599 .OR. second.LT.0) ierr=1
         if(ierr.NE.0) then
            write(iolst,*)'ERROR in DATE_HRSEC -- Invalid inputs'
            write(iolst,*)'      YEAR =',year
            write(iolst,*)'JULIAN DAY =',juld
            write(iolst,*)'      HOUR =',hour
            write(iolst,*)'    SECOND =',second
            return
         endif
      elseif(action.EQ.'TOINDV') then
         lbreakup=.TRUE.
      else
         write(iolst,*)'ERROR in DATE_HRSEC -- Invalid ACTION'
         write(iolst,*)'   ACTION expected: TOINDV or TOSNGL'
         write(iolst,*)'   ACTION found   : ',action
         ierr=1
         return
      endif

      if(lbreakup) then
c ---    Break long integer into components
         year=datetime/1000000000_8
         juld=datetime/1000000_8-year*1000_8
         hour=datetime/10000_8-year*100000_8-juld*100_8
         second=datetime-year*1000000000_8-juld*1000000_8-hour*10000_8
      else
c ---    Combine components into long integer
         datetime=year*1000000000_8+juld*1000000_8+hour*10000_8+second
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine date_hr(iolst,action,year,juld,hour,
     &                   datehour,ierr)
c----------------------------------------------------------------------
c --- UTILITY   Version: 1.1            Level: 120518          DATE_HR
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Converts between individual date and time variables
c               and single integer YYYYJJJHH variable
c
c --- INPUT:
c               IOLST - integer   - Unit number of output list file
c              ACTION - char*     - Action to perform
c                                   TOINDV returns components
c                                   TOSNGL returns single integer
c
c --- OUTPUT:
c                IERR - integer   - Error flag (0 = No Error)
c
c --- INPUT/OUTPUT:
c                YEAR - integer   - Year (YYYY)
c                JULD - integer   - Julian day (JJJ)
c                HOUR - integer   - Hour (HH)
c            DATEHOUR - integer   - Combined date & time (YYYYJJJHH)
c
c --- DATE_HR called by:  ()
c --- DATE_HR calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: action
      integer          :: iolst,year,juld,hour,datehour,ierr

c --- Locals
      logical  :: lbreakup

c --- Initializations
      ierr=0

c --- Set conversion
      if(action.EQ.'TOSNGL') then
         lbreakup=.FALSE.
c ---    QA inputs
         if(year.GT.9999 .OR. year.LT.1) ierr=1
         if(juld.GT.366 .OR. juld.LT.1) ierr=1
         if(hour.GT.24 .OR. hour.LT.0) ierr=1
         if(ierr.NE.0) then
            write(iolst,*)'ERROR in DATE_HRSEC -- Invalid inputs'
            write(iolst,*)'      YEAR =',year
            write(iolst,*)'JULIAN DAY =',juld
            write(iolst,*)'      HOUR =',hour
            return
         endif
      elseif(action.EQ.'TOINDV') then
         lbreakup=.TRUE.
      else
         write(iolst,*)'ERROR in DATE_HRSEC -- Invalid ACTION'
         write(iolst,*)'   ACTION expected: TOINDV or TOSNGL'
         write(iolst,*)'   ACTION found   : ',action
         ierr=1
         return
      endif

      if(lbreakup) then
c ---    Break integer into components
         year=datehour/100000
         juld=datehour/100-year*1000
         hour=datehour-year*100000-juld*100
      else
c ---    Combine components into single integer
         datehour=year*100000+juld*100+hour
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine cal_day(iolst,action,year,month,day,julday,ierr)
c----------------------------------------------------------------------
c --- UTILITY   Version: 1.1            Level: 120518          CAL_DAY
c               J. Scire, Exponent Inc.
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Convert between Julian day and Gregorian date
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c              ACTION - char*     - Action to perform
c                                   TOJULIAN returns Julian day
c                                   TOGREGOR returns Gregorian date
c
c --- OUTPUT:
c                IERR - integer   - Error flag (0 = No Error)
c
c --- INPUT/OUTPUT:
c                YEAR - integer   - Year (YYYY)
c               MONTH - integer   - Month (MM)
c                 DAY - integer   - Day (DD)
c              JULDAY - integer   - Julian day (JJJ)
c
c --- CAL_DAY called by:  ()
c --- CAL_DAY calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      character(len=*) :: action
      integer          :: iolst,year,month,day,julday,ierr

c --- Locals
      logical  :: ljulian
      integer  :: i,ndpmo(12,2),cumday(12,2),leap,lastday(2)

c --- Set number of days in each month (regular,leap)
      data ndpmo/31,28,31,30,31,30,31,31,30,31,30,31,
     &           31,29,31,30,31,30,31,31,30,31,30,31/

c --- Set number of days in year (regular,leap)
      data lastday/365,366/

c --- Set cumulative number of days to start of each
c --- month (regular,leap)
      data cumday/0,31,59,90,120,151,181,212,243,273,304,334,
     &            0,31,60,91,121,152,182,213,244,274,305,335/

c --- Initializations
      ierr=0

c --- Set leap year index (1=regular, 2=leap)
      leap=1
      if(MOD(year,4).EQ.0) leap=2

c --- Set conversion
      if(action.EQ.'TOJULIAN') then
         ljulian=.TRUE.
c ---    QA inputs
         if(month.GT.12 .OR. month.LT.1) ierr=1
         if(ierr.EQ.0) then
            if(day.LT.1 .OR. day.GT.ndpmo(month,leap)) ierr=1
         endif
         if(ierr.NE.0) then
            write(iolst,*)'ERROR in CAL_DAY -- Invalid inputs'
            write(iolst,*)'      YEAR =',year
            write(iolst,*)'     MONTH =',month
            write(iolst,*)'       DAY =',day
            return
         endif
      elseif(action.EQ.'TOGREGOR') then
         ljulian=.FALSE.
c ---    QA inputs
         if(julday.GT.lastday(leap) .OR. julday.LT.1) ierr=1
         if(ierr.NE.0) then
            write(iolst,*)'ERROR in CAL_DAY -- Invalid inputs'
            write(iolst,*)'      YEAR =',year
            write(iolst,*)'JULIAN DAY =',julday
            return
         endif
      else
         write(iolst,*)'ERROR in CAL_DAY -- Invalid ACTION'
         write(iolst,*)'   ACTION expected: TOJULIAN or TOGREGOR'
         write(iolst,*)'   ACTION found   : ',action
         ierr=1
         return
      endif

      if(ljulian) then
c ---    Convert date to the Julian day
         julday=cumday(month,leap)+day
      else
c ---    Convert Julian day to the date
         if(julday.LE.cumday(7,leap)) then
            do i=6,1,-1
               if(julday.GT.cumday(i,leap)) then
                  month=i
                  day=julday-cumday(i,leap)
                  return
               endif
            enddo
         else
            do i=12,7,-1
               if(julday.GT.cumday(i,leap)) then
                  month=i
                  day=julday-cumday(i,leap)
                  return
               endif
            enddo
         endif
      endif

      return
      end

c----------------------------------------------------------------------
      subroutine time_incr(iolst,year,julday,hour,second,dthr,dtsec,
     &                     ierr)
c----------------------------------------------------------------------
c --- UTILITY   Version: 1.1            Level: 120518        TIME_INCR
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Add time interval (pos or neg) to a date-time
c               Date (Julian day) is updated accordingly
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c                DTHR - integer   - Hours to ADD to time (may be <0)
c               DTSEC - integer   - Seconds to ADD to time (may be <0)
c
c --- OUTPUT:
c                IERR - integer   - Error flag (0 = No Error)
c
c --- INPUT/OUTPUT:
c                YEAR - integer   - Year (YYYY)
c              JULDAY - integer   - Julian day (JJJ)
c                HOUR - integer   - Hour (HH)
c              SECOND - integer   - Second (SSSS)
c
c --- TIME_INCR called by:  ()
c --- TIME_INCR calls:      none
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      integer          :: iolst,year,julday,hour,second
      integer          :: dthr,dtsec,ierr

c --- Locals
      integer          :: dh,ds,dtday,lastday

c --- Initializations
      ierr=0
      dtday=0

c --- Incremental hours and seconds must have the same sign
c --- (product must be zero or positive)
      if((dthr*dtsec) .LT. 0.0) then
         ierr=1
         write(iolst,*)'ERROR in TIME_INCR -- Invalid increment'
         write(iolst,*)'Hours and seconds must have same sign'
         write(iolst,*)'DTHR, DTSEC =',dthr,dtsec
         return
      endif

c --- Condition increment so that there are fewer than 3600 seconds
      dh=dthr
      ds=dtsec
      if(ABS(ds).GT.3599) then
         dh=dh+ds/3600
         ds=MOD(ds,3600)
      endif

c --- Apply change in seconds
      second=second+ds
      if(second.LT.0) then
c ---    Move hours back 1 hour and seconds forward 3600
         dh=dh-1
         second=second+3600
      elseif(second.GE.3600) then
c ---    Move hours forward 1 hour and seconds back 3600
         dh=dh+1
         second=second-3600
      endif

c --- Condition increment so that there are fewer than 24 hours
      if(ABS(dh).GT.24) then
         dtday=dh/24
         dh=MOD(dh,24)
      endif

c --- Apply change in hours
      hour=hour+dh
      if(hour.LT.0) then
c ---    Move days back 1 day and hours forward 24
         dtday=dtday-1
         hour=hour+24
      elseif(hour.GE.24) then
c ---    Move days forward 1 day and hours back 24
         dtday=dtday+1
         hour=hour-24
      endif

c --- Apply change in days
      julday=julday+dtday

      if(julday.GT.0) then
c ---    Current year or beyond
10       lastday=365
         if(MOD(year,4).EQ.0) lastday=366
         if(julday.GT.lastday) then
c ---       Move years forward 1 year and days back
            julday=julday-lastday
            year=year+1
         else
            return
         endif
         goto 10
      else
c ---    Previous year or beyond
c ---    Move years back and days forward
20       year=year-1
         lastday=365
         if(MOD(year,4).EQ.0) lastday=366
         julday=julday+lastday
         if(julday.GT.0) return
         goto 20
      endif

      return
      end


c----------------------------------------------------------------------
      subroutine time_diff(iolst,time1,time2,delday,delhour,delsecond,
     &                     ierr)
c----------------------------------------------------------------------
c --- UTILITY   Version: 1.1            Level: 120518        TIME_DIFF
c               D. Strimaitis, Exponent Inc.
c
c --- PURPOSE:  Compute time interval (time2-time1)
c
c --- INPUTS:
c               IOLST - integer   - Unit number of output list file
c               TIME1 - integer*8 - Period start (YYYYJJJHHSSSS)
c               TIME2 - integer*8 - Period end   (YYYYJJJHHSSSS)
c
c --- OUTPUT:
c              delday - integer   - Time difference (whole days)
c             delhour - integer   - Time difference (whole hours)
c           delsecond - integer   - Time difference (whole seconds)
c                IERR - integer   - Error flag (0 = No Error)
c
c --- TIME_DIFF called by:  ()
c --- TIME_DIFF calls:      DATE_HRSEC
c----------------------------------------------------------------------
      implicit none

c --- Declare arguments
      integer(kind=8)  :: time1,time2,diff
      integer          :: iolst,delday,delhour,delsecond,ierr

c --- Locals
      logical          :: lswap
      integer          :: year1,jul1,hour1,sec1
      integer          :: year2,jul2,hour2,sec2
      integer          :: i

c --- Initializations
      ierr=0
      delday=0
      delhour=0
      delsecond=0

c --- Simple difference of times
      diff=time2-time1

      if(ABS(diff).LE.3600_8) then
c ---    Less than 1 hour difference
         delsecond=diff
         return
      endif

c --- Work with positive time differences
      lswap=.FALSE.
      if(diff.LT.0_8) then
         lswap=.TRUE.
      endif

      if(lswap) then
         call DATE_HRSEC(iolst,'TOINDV',year1,jul1,hour1,sec1,
     &                   time2,ierr)
         call DATE_HRSEC(iolst,'TOINDV',year2,jul2,hour2,sec2,
     &                   time1,ierr)
      else
         call DATE_HRSEC(iolst,'TOINDV',year1,jul1,hour1,sec1,
     &                   time1,ierr)
         call DATE_HRSEC(iolst,'TOINDV',year2,jul2,hour2,sec2,
     &                   time2,ierr)
      endif

c --- Difference in whole days
      if(year2.NE.year1) then
c ---    Step through each year summing the days
         do i=year1,year2
            if(i.EQ.year1) then
               delday=365-jul1
               if(MOD(year1,4).EQ.0) delday=delday+1
            elseif(i.EQ.year2) then
               delday=delday+jul2
            else
               delday=delday+365
               if(MOD(i,4).EQ.0) delday=delday+1
            endif
         enddo
      else
         delday=jul2-jul1
      endif

c --- Difference in whole hours
      delhour=hour2-hour1
      if(delhour.LT.0) then
c ---    Add 24 hours and remove 1 day
         delhour=delhour+24
         delday=delday-1
      endif

c --- Difference in whole seconds
      delsecond=sec2-sec1
      if(delsecond.LT.0) then
c ---    Add 3600 seconds and remove 1 hour
         delsecond=delsecond+3600
         delhour=delhour-1
      endif

      if(lswap) then
         delday=-delday
         delhour=-delhour
         delsecond=-delsecond
      endif

      return
      end
